/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/
#include <QCoreApplication>
#include <QDir>
#include <QFileInfo>
#include <QSettings>
#include <QMessageBox>
#include <QDebug>

#ifdef Q_WS_X11
#include <QProcess>
//#include <plasma/wallpaper.h>
#endif

#ifdef Q_WS_WIN
  #include <fstream>
  #include <windows.h>
#endif

#ifdef Q_WS_PM
  #include <os2.h>
  #include <os2emx.h>
#endif

#include "mainwindow.h"
#include "backgroudimage.h"
#include "network.h"

#ifdef Q_WS_MAEMO_5
#include <QApplication>
#include <QDesktopWidget>
extern "C"
{
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
}
#endif

QString backgroudImage::windowManager="";

backgroudImage::backgroudImage(const QString &url )
{
    QByteArray b;
    QPixmap pix;
    QImage img;

    if (windowManager=="") detectWM();
    if (windowManager=="") {
        QMessageBox::critical(0,QObject::tr("error"),QObject::tr("Window manager not detected") );
        return;
    }
    if (windowManager=="KDE4")
    {
//        TODO
//        dunno how to change wallpaper here
        QMessageBox::critical(0,QObject::tr("error"),QObject::tr("Can't set wallpaper on Kde4 yet") );
        return;
    }

#ifdef Q_WS_PM
    // remove background
    HOBJECT hobj;
    const char *objName = "<WP_DESKTOP>";

    if (hobj = WinQueryObject(reinterpret_cast< const unsigned char * > (objName)))
        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > ((QString("BACKGROUND=\"\",N,1,I;")).toAscii().constData()));
#endif

    network *net_async=new network(false, MainWindow::pointerToMainWindow);
    b=net_async->get_synchronous(url,false);
    if (!b.isEmpty())
    {
        pix.loadFromData(b);
        img=pix.toImage();
    }
    net_async->deleteLater();
    net_async=0;

    if (!b.isEmpty())
        setBackground(img);
    else
        QMessageBox::critical(0,QObject::tr("download wallpaper error"),QObject::tr("Can't download: ")+url,QMessageBox::Ok );
}

// Set wallpaper accordling to OS
void backgroudImage::setBackground(QImage &image)
{
    QString imageCompletePath;
    QStringList args;

#if defined(Q_WS_X11) && !defined(Q_WS_MAEMO_5)
    if (windowManager=="GNOME")
    {
        imageCompletePath=QDir::homePath()+"/MDwallpaper.jpg";
        image.save(imageCompletePath,"JPG");
        args << "--type" << "bool" << "--set" << "/desktop/gnome/background/draw_background" << "true";
        QProcess::execute("gconftool-2",args);
        args.clear();
        args << "--type" << "string" << "--set" << "/desktop/gnome/background/picture_options" << "scaled";
        QProcess::execute("gconftool-2",args);
        args.clear();
        args << "--type" << "string" << "--set" <<
                "/desktop/gnome/background/picture_filename" << imageCompletePath;
        QProcess::execute("gconftool-2",args);
    }

    if (windowManager=="KDE4")
    {
//        TODO
//        dunno how to change wallpaper here
    }

    if (windowManager=="KDE3")
    {
        imageCompletePath=QDir::homePath()+"/MDwallpaper.jpg";
        image.save(imageCompletePath,"JPG");
        args << "kdesktop" << "KBackgroundIface" << "setWallpaper"
                << imageCompletePath << "1";
        QProcess::execute("dcop",args);
    }
#endif

#ifdef Q_WS_MAEMO_5
    // If the image is a vertical image ask to rotate it
    if (image.width() < image.height()){
        QMessageBox msgBox;
        msgBox.setText(QObject::tr("Would you like to rotate anticlockwise ?"));
        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        msgBox.setDefaultButton(QMessageBox::Yes);
        if (msgBox.exec()==QMessageBox::Yes)
            image=image.transformed(QMatrix().rotate(-90));
    }

    QSize desktopSize = QApplication::desktop()->size();
    image = image.scaled(desktopSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    image = inscribeImageIntoDesktopSize(desktopSize, image);
    // Return values for ask_which_desktop_dialog():
    // -1   = all
    // 0    = current
    // 1..4 = desktop number
    int n_desktop = ask_which_desktop_dialog();
    int currentDesktop;
    switch (n_desktop) {
    case -1:
        imageCompletePath="/home/user/MyDocs/.images/MDwallpaper.jpg";
        image.save(imageCompletePath,"JPG");
        setMaemoWallpaper(0, imageCompletePath);
        setMaemoWallpaper(1, imageCompletePath);
        setMaemoWallpaper(2, imageCompletePath);
        setMaemoWallpaper(3, imageCompletePath);
        break;
    case 0:
        // Get current desktop
        currentDesktop = getXIntProperty("_NET_CURRENT_DESKTOP");
        // Save image
        imageCompletePath="/home/user/MyDocs/.images/MDwallpaper"+QString::number(1 + currentDesktop)+".jpg";
        image.save(imageCompletePath,"JPG");
        setMaemoWallpaper(currentDesktop, imageCompletePath);
        break;
    case 1:
    case 2:
    case 3:
    case 4:
        // Save image
        imageCompletePath="/home/user/MyDocs/.images/MDwallpaper"+QString::number(n_desktop)+".jpg";
        image.save(imageCompletePath,"JPG");
        setMaemoWallpaper(n_desktop-1, imageCompletePath);
        break;
    }
    qDebug() << "image saved in" << imageCompletePath;
#endif  // Q_WS_MAEMO_5

#ifdef Q_WS_WIN
    imageCompletePath=QDir::homePath()+"/MDwallpaper.bmp";
    image.save(imageCompletePath,"BMP");

    QSettings regEdit("HKEY_CURRENT_USER\\Control Panel\\Desktop",QSettings::NativeFormat);
    regEdit.setValue("Wallpaper",QDir::toNativeSeparators(imageCompletePath));
    regEdit.setValue("WallpaperStyle","0");
    regEdit.setValue("TileWallpaper","0");
    QString s = QDir::toNativeSeparators(imageCompletePath.toAscii());
    bool b=SystemParametersInfoA(SPI_SETDESKWALLPAPER,0,
                          reinterpret_cast<void *>
                          (QDir::toNativeSeparators(imageCompletePath).toAscii().data()),
                          SPIF_SENDCHANGE);//SPIF_SENDWININICHANGE);
    qDebug() << "Wallpaper updated:" << b;
#endif  // Q_WS_WIN

#ifdef Q_WS_PM
    imageCompletePath=QDir::homePath()+"/MDwallpaper.bmp";
    QFile::remove(imageCompletePath);
    image.save(imageCompletePath,"BMP");

    HOBJECT hobj;
    const char *objName = "<WP_DESKTOP>";

    if (hobj = WinQueryObject(reinterpret_cast< const unsigned char * > (objName)))
//        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > ((QString().arg(QDir::toNativeSeparators(imageCompletePath))).toAscii().constData()));
        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > (QDir::toNativeSeparators(imageCompletePath).toAscii().constData()));
#endif  // Q_WS_PM
}


QString backgroudImage::detectWM()
{
#if defined(Q_WS_X11) && !defined(Q_WS_MAEMO_5)
    QList<QFileInfo> files = QDir("/proc").entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);
    QFileInfo file;
    int i;
    QString s;

    i=0;
    while (i<files.size())
    {
        qDebug() << QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
        if (QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit") &&
            !QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("start_kdeinit") &&
            !QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit4"))
        {
            s=QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
            windowManager="KDE3";
            break;
        }
        if (QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("kdeinit4"))
        {
            s=QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
            windowManager="KDE4";
            break;
        }
        if (QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget().contains("gconfd-2"))
        { windowManager="GNOME"; break; }
        qDebug() << QFileInfo(files.at(i).absoluteFilePath() + "/exe").symLinkTarget();
        i++;
    }
#endif

#ifdef Q_WS_MAEMO_5
    windowManager="MAEMO_5";
#endif

#ifdef Q_WS_WIN
    windowManager="WINDOWS";
#endif

#ifdef Q_WS_MAC
    windowManager="MAC";
#endif

#ifdef Q_WS_PM
    windowManager="OS/2";
#endif

    qDebug() << "Windows manager detected: " << windowManager;
    return windowManager;
}


///////////////////////////////////////////////////////
// Maemo specific functions
#ifdef Q_WS_MAEMO_5

// Return values:
// -1   = all
// 0    = current
// 1..4 = desktop number
int backgroudImage::ask_which_desktop_dialog()
{
    QMessageBox msgBox;
    QPushButton *all, *current, *first, *second, *third, *fourth;
    msgBox.setText(QObject::tr("Choose in which desktop to set wallpaper"));
    msgBox.setStandardButtons(QMessageBox::Cancel);

    all     = msgBox.addButton("All", QMessageBox::AcceptRole);
    current = msgBox.addButton("Current: "+QString::number(getXIntProperty("_NET_CURRENT_DESKTOP")+1), QMessageBox::AcceptRole);
    first   = msgBox.addButton("1", QMessageBox::AcceptRole);
    second  = msgBox.addButton("2", QMessageBox::AcceptRole);
    third   = msgBox.addButton("3", QMessageBox::AcceptRole);
    fourth  = msgBox.addButton("4", QMessageBox::AcceptRole);

    msgBox.setDefaultButton(QMessageBox::Cancel);
    int ret = msgBox.exec();
    return ret-1;
}

void backgroudImage::setMaemoWallpaper(int desktop, QString imageFilePath)
{
    QStringList args;
    args << "-s" << "--type" << "string" << "/apps/osso/hildon-desktop/views/"+QString::number(desktop+1)+"/bg-image" << "imageFilePath";
    QProcess::execute("gconftool",args);

    args.clear();
    args << "-s" << "--type" << "string" << "/apps/osso/hildon-desktop/views/"+QString::number(desktop+1)+"/bg-image" << imageFilePath;
    QProcess::execute("gconftool",args);

    qDebug () << "gconftool" << args;

//    GConfClient *client = gconf_client_get_default();
//    if (client) {
//        GError *error = 0;
//        char  *path = g_strdup_printf("/apps/osso/hildon-desktop/views/%i/bg-image",
//                                      1 + desktop);
//        // call gconf_client_set_string() first with an empty string because if the
//        // wallpaper image name doesn't change, wallpaper isn't updated
//        gconf_client_set_string(client, path, "", &error);
//        QStringList args;
//        args << "--type" << "string" << "--set" << "/desktop/gnome/background/picture_options" << "scaled";
//        QProcess::execute("gconftool-2",args);
//        if (!gconf_client_set_string(client, path, imageFilePath.toUtf8().data(), &error)) {
//            qDebug() << imageFilePath.toUtf8().data() << error->message;
//            g_error_free(error);
//        }
//        g_free(path);
//        g_object_unref(client);
//    } else
//        qDebug("Cannot get GConf2 client");
}

// from http://wiki.forum.nokia.com/index.php/How_to_set_wallpaper_in_Maemo_5
// Function for obtaining number of currently active desktop:
int backgroudImage::getXIntProperty(const char *propertyName)
{
    Atom realType;
    int format, status, num;
    unsigned long n, extra;
    unsigned char *data = 0;
    char *display_name = getenv("DISPLAY");
    Display *display = XOpenDisplay(display_name);
    int screen = DefaultScreen(display);
    Window rootWindow = RootWindow(display, screen);
    Atom atom = XInternAtom(display, propertyName, false);

    status = XGetWindowProperty(display, rootWindow, atom, 0L, 16L,
                        0, XA_CARDINAL, &realType, &format,
                        &n, &extra, &data);
    if (status == Success && realType == XA_CARDINAL &&
        format == 32 && n == 1 && data)
        num = ((int*)data)[0];
    else
        num = -1;

    if (data)
        XFree(data);
    XCloseDisplay(display);

    return num;
}

QImage backgroudImage::inscribeImageIntoDesktopSize(QSize desktopSize, QImage image)
{
    QImage ret(desktopSize, image.format());
    int x, y, padX, padY;
    int imageBytesLine;
    int retBytesLine;
    uchar *imagePtr;
    uchar *retPtr;

    imagePtr       = image.bits();
    retPtr         = ret.bits();
    imageBytesLine = image.bytesPerLine();
    retBytesLine   = ret.bytesPerLine();

    if (desktopSize.width() > image.width()) {
        padX = ( desktopSize.width() - image.width() ) / 2 * (image.depth()>>3);
        if (padX<0) padX=0;
        for (y=0; y<image.height(); y++)
        {
            memcpy(retPtr+padX+(y*retBytesLine),
                   imagePtr+(y*imageBytesLine),
                   imageBytesLine);
        }
    }
    else {
        padY = ( desktopSize.height() - image.height() ) / 2 * retBytesLine;
        if (padY<0) padY=0;
        for (y=0; y<image.height(); y++)
        {
            qDebug() << y;
            memcpy(retPtr+padY+(y*retBytesLine),
                   imagePtr+(y*imageBytesLine),
                   imageBytesLine);
        }
    }
    return ret;
}

#endif // Q_WS_MAEMO_5
