/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QWebView>
#include <QWebPage>
#include <QWebFrame>
#include "browse.h"
#include "ui_browse.h"

#include <QWebPluginFactory>
#include <QDebug>

//class	ExtensionOption;
//class	ExtensionReturn;
//class	MimeType;
//class	Plugin;

browse::browse(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::browse)
{
    ui->setupUi(this);
    ui->webView->setMaximumSize(QSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX));

    QWebSettings::globalSettings()->setAttribute(QWebSettings::PluginsEnabled, true);
    zoom=1;
    step=0.05;
}

browse::~browse()
{
    delete ui;
}

void browse::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

int browse::get_scroolbar_value()
{
    return ui->webView->page()->mainFrame()->scrollBarValue(Qt::Vertical);
}

void browse::set_scroolbar_value(int n)
{
    ui->webView->page()->mainFrame()->setScrollBarValue(Qt::Vertical,n);
}

void browse::stopLoading()
{
    ui->webView->stop();
}

void browse::on_PB_close_clicked()
{
    emit button_close_pressed();
}

void browse::setUrl(QString url)
{
    ui->webView->setUrl(url);
}

void browse::setHtml(QString html)
{
    ui->webView->setHtml(html);
}

void browse::on_webView_urlChanged(QUrl url)
{
    ui->LE_url->setText(url.toString());
}

void browse::on_LE_url_returnPressed()
{
    ui->webView->setUrl(ui->LE_url->text());
}

void browse::on_PB_right_clicked()
{
    ui->webView->forward();
    ui->LE_url->setText(ui->webView->url().toString());
}

void browse::on_PB_left_clicked()
{
    ui->webView->back();
    ui->LE_url->setText(ui->webView->url().toString());
}

void browse::on_PB_zoomin_clicked()
{
    if (zoom+step>0) zoom-=step;
    ui->webView->setZoomFactor(zoom);
}

void browse::on_PB_zoomout_clicked()
{
    if (zoom+step>0) zoom+=step;
    ui->webView->setZoomFactor(zoom);
}

