/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QColorDialog>
#include <QFileDialog>
#include <QMessageBox>
#include "customizeimagewidget.h"
#include "ui_customizeimagewidget.h"
#include "settings.h"

customizeImageWidget::customizeImageWidget(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::customizeImageWidget)
{
    ui->setupUi(this);

    connect(ui->PB_choose_top_color, SIGNAL(clicked()), this, SLOT(onPB_choose_top_color_clicked()));
    connect(ui->PB_choose_mid_color, SIGNAL(clicked()), this, SLOT(onPB_choose_mid_color_clicked()));
    connect(ui->PB_choose_bottom_color, SIGNAL(clicked()), this, SLOT(onPB_choose_bottom_color_clicked()));
    connect(ui->PB_images_color, SIGNAL(clicked()), this, SLOT(onPB_images_color_clicked()));
    connect(ui->PB_videos_color, SIGNAL(clicked()), this, SLOT(onPB_videos_color_clicked()));
    connect(ui->PB_images_text_color, SIGNAL(clicked()), this, SLOT(onPB_images_text_color_clicked()));
    connect(ui->PB_videos_text_color, SIGNAL(clicked()), this, SLOT(onPB_videos_text_color_clicked()));
}

customizeImageWidget::~customizeImageWidget()
{
    settings conf;
    conf.store_settings("graphicsView/image_path", ui->LE_image->text());
    conf.store_settings("graphicsView/use_background_image", ui->CB_use_background_image->isChecked());
    delete ui;
}

void customizeImageWidget::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void customizeImageWidget::initDialog(QColor top, QColor mid, QColor bottom, QColor images, QColor videos, QColor images_text, QColor videos_text)
{
    settings conf;

    top_color    = top;
    mid_color    = mid;
    bottom_color = bottom;
    images_color = images;
    videos_color = videos;
    images_text_color   = images_text;
    videos_text_color   = videos_text;

    ui->LE_image->setText( conf.conf->value("graphicsView/image_path",QString("")).toString() );
    ui->CB_use_background_image->setChecked(  conf.conf->value("graphicsView/use_background_image",true).toBool());

    refreshLabelsColor();
}

void customizeImageWidget::refreshLabelsColor()
{
    QString s="background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1,stop:0 rgba("+
      QString::number(top_color.red())+","+
      QString::number(top_color.green())+","+
      QString::number(top_color.blue())+","+
      QString::number(top_color.alpha())+
      "),"+
      "stop:0.5 rgba("+
      QString::number(mid_color.red())+","+
      QString::number(mid_color.green())+","+
      QString::number(mid_color.blue())+","+
      QString::number(mid_color.alpha())+
      "),"+
      "stop:1 rgba("+
      QString::number(bottom_color.red())+","+
      QString::number(bottom_color.green())+","+
      QString::number(bottom_color.blue())+","+
      QString::number(bottom_color.alpha())+
      "));";
    ui->L_gradient_colors->setStyleSheet(s);

    s="background-color: rgb("+
          QString::number(images_color.red())+","+
          QString::number(images_color.green())+","+
          QString::number(images_color.blue())+");";
    ui->L_images_color->setStyleSheet(s);
    s="background-color: rgb("+
          QString::number(videos_color.red())+","+
          QString::number(videos_color.green())+","+
          QString::number(videos_color.blue())+");";
    ui->L_videos_color->setStyleSheet(s);
    s="background-color: rgb("+
          QString::number(images_text_color.red())+","+
          QString::number(images_text_color.green())+","+
          QString::number(images_text_color.blue())+");";
    ui->L_images_text_color->setStyleSheet(s);
    s="background-color: rgb("+
          QString::number(videos_text_color.red())+","+
          QString::number(videos_text_color.green())+","+
          QString::number(videos_text_color.blue())+");";
    ui->L_videos_text_color->setStyleSheet(s);
}

void customizeImageWidget::onPB_choose_top_color_clicked()
{
    QColor c = QColorDialog::getColor(top_color,this,"",QColorDialog::ShowAlphaChannel);
    if (c.isValid()) {
        emit top_color_changed(c);
        top_color=c;
        refreshLabelsColor();
    }
}

void customizeImageWidget::onPB_choose_mid_color_clicked()
{
    QColor c = QColorDialog::getColor(mid_color,this,"",QColorDialog::ShowAlphaChannel);
    if (c.isValid()) {
        emit mid_color_changed(c);
        mid_color=c;
        refreshLabelsColor();
    }
}

void customizeImageWidget::onPB_choose_bottom_color_clicked()
{
    QColor c = QColorDialog::getColor(bottom_color,this,"",QColorDialog::ShowAlphaChannel);
    if (c.isValid()) {
        emit bottom_color_changed(c);
        bottom_color=c;
        refreshLabelsColor();
    }
}

void customizeImageWidget::on_PB_choose_image_clicked()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::AnyFile);
    dialog.setDirectory(QDir(ui->LE_image->text()));
//    dialog.setNameFilter(tr("Media Downloader file (*.md)"));
    dialog.setViewMode(QFileDialog::Detail);
//    dialog.setLabelText(QFileDialog::FileName);

    if (dialog.exec())
    {
        QString s=dialog.selectedFiles().at(0);
        if (!s.isEmpty()) {
            ui->LE_image->setText(s);
            emit image_choosen(ui->CB_use_background_image->isChecked(), s);
        }
    }
}

void customizeImageWidget::on_CB_use_background_image_toggled(bool checked)
{
    emit image_choosen(checked, ui->LE_image->text());
}

void customizeImageWidget::onPB_images_color_clicked()
{
    QColor c = QColorDialog::getColor(images_color,this,"");
    if (c.isValid()) {
        emit images_color_changed(c);
        images_color=c;
        refreshLabelsColor();
    }
}

void customizeImageWidget::onPB_videos_color_clicked()
{
    QColor c = QColorDialog::getColor(videos_color,this,"");
    if (c.isValid()) {
        emit videos_color_changed(c);
        videos_color=c;
        refreshLabelsColor();
    }
}

void customizeImageWidget::onPB_images_text_color_clicked()
{
    QColor c = QColorDialog::getColor(images_text_color,this,"");
    if (c.isValid()) {
        emit images_text_color_changed(c);
        images_text_color=c;
        refreshLabelsColor();
    }
}

void customizeImageWidget::onPB_videos_text_color_clicked()
{
    QColor c = QColorDialog::getColor(videos_text_color,this,"");
    if (c.isValid()) {
        emit videos_text_color_changed(c);
        videos_text_color=c;
        refreshLabelsColor();
    }
}
