/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QDebug>
#include "googlepicture.h"

const char *GooglePicture::content_type[] = {
    "",
    "news",
    "face",
    "photo",
    "clipart",
    "lineart"
};
const char *GooglePicture::image_size[] = {
    "",
    "m",
    "l",
    "i",
    "qsvga",
    "vga",
    "svga",
    "xga",
    "2mp",
    "4mp",
    "6mp",
    "8mp",
    "10mp",
    "12mp",
    "15mp",
    "20mp",
    "40mp",
    "70mp"
};
const char *GooglePicture::coloration[] = {
    "",
    "gray",
    "color"
};
const char *GooglePicture::safe_filter[] = {
    "off",
    "images",
    "active"
};
const char *GooglePicture::content_rights[] = {
    "",
    "f",
    "fc",
    "fm",
    "fmc"
};



GooglePicture::GooglePicture(parse_method method, QObject *parent) : QObject(parent)
{
    parse=method;
    net_thumbs = new network(false);
    connect(net_thumbs, SIGNAL(downloadFinished(int,QByteArray)),
            this, SLOT(gotImageSlot(int,QByteArray)));
    connect(net_thumbs, SIGNAL(queue_completed()),
            this, SIGNAL(googleThumbsGetCompleted()));

}
GooglePicture::~GooglePicture()
{
    net_thumbs->deleteLater();
}
//////////////////////////////////////////////////////////////
void GooglePicture::set_parse_method(parse_method method)
{
    parse=method;
}

int GooglePicture::get_images_list(QString search_string,
                                   int start_image_number, int images_to_get,
                                   QString content_type,
                                   QString image_size,
                                   QString coloration,
                                   QString safeFilter,
                                   QString rights,
                                   QString siteSearch)
{
    QString google_url;
    // greek search example: καλαμάτα
    // russian search example: русская водка
    // now seems to work without .toLocal8Bit unlike early version
    search_string=search_string.replace(" ","+");//.toLocal8Bit();

    QByteArray data;
    int res=-1;
    tot_found=0;
    currentSearchString = search_string;

    net_search = new network(false);
    estimatedResultCount=0;
    while (tot_found<images_to_get && res!=0)
    {
        if (parse==METHOD_AJAX) {
            google_url="http://ajax.googleapis.com/ajax/services/search/images?v=1.0&";
            google_url+="q="+search_string+\
                        "&start="+QString::number(start_image_number+tot_found)+\
                        "&imgtype="+content_type+\
                        "&imgsz="+image_size+\
                        "&imgc="+coloration+\
                        "&safe="+safeFilter+\
                        "&as_rights="+rights+\
                        "&as_sitesearch="+siteSearch;
        }
        else {  // http://www.google.com/search?hl=en&safe=off &client=ubuntu &channel=fs &biw=1531 &bih=89 0&q=ciao &gs_sm=e &gs_upl=15054l15517l0l4l4l0l0l0l0l230l693l0.2.2l4 &um=1 &ie=UTF-8 &tbm=isch &source=og &sa=N &tab=wi
            google_url="http://images.google.com/images?";
            google_url+="as_q="+search_string+\
                        "&start="+QString::number(start_image_number+tot_found)+\
                        "&as_sitesearch="+siteSearch+\
                        "&safe="+safeFilter+\
                        "&btnG=Google+Search";
            QString tbs;
            tbs="&tbs=isch:1";
            if (!content_type.isEmpty()) tbs+=",itp:"+content_type;
            if (!image_size.isEmpty())   tbs+=",isz:lt,islt:"+image_size;
            if (!coloration.isEmpty())   tbs+=",ic:"+coloration;
            if (!rights.isEmpty())       tbs+=",iur:"+rights;
            google_url+=tbs;
        }

        data=net_search->get_synchronous(google_url,false);
        if (data!="") {
            if (parse==METHOD_AJAX) res=ajax_parseResult(data,images_to_get);
            else res=public_parseResult(data,images_to_get);
        }
        else res=0; // error retrieving google list from the net
        tot_found+=res;
        emit this->outMessage(tr("Images parsed: ") + QString::number(tot_found),2);
        emit this->outMessage("",1);
    }
    delete net_search;
    return tot_found;
}

// Return a list of all images url
QStringList GooglePicture::getImagesUrl()
{
    QStringList l;
    for (int i=0; i<images.size(); i++) l.append(images.at(i).url);
    return l;
}

// Get images from filled QList<GoogleImageStruct> images
void GooglePicture::getThumbsImages()
{
    int i;

    i=0;
    while (i<images.size())
    {
//        qDebug() << "get thumb: " << images.at(i).tbUrl;
        net_thumbs->get(images.at(i).tbUrl);
        i++;
    }
}
void GooglePicture::gotImageSlot(const int ID, const QByteArray &data)
{
    emit gotImageSignal(data, images.at(ID));
}
void GooglePicture::queueCompleted()
{
    emit googleThumbsGetCompleted();
}
//////////////////////////////////////////////////////////////
// FUNCTIONS TO PARSE AJAX SEARCH RESULTS ////////////////////
//////////////////////////////////////////////////////////////
// With my great disappointment, I found that ajax search
// is limited to return at last 64 results. So back to implement
// regExps to parse on the public images html search page
//////////////////////////////////////////////////////////////
// Only 1st captured text is returned
QString GooglePicture::getRegExp(QString pattern, QString str)
{
    QRegExp rx;
    rx.setMinimal(true);
    rx.setPattern(pattern);
    rx.indexIn(str);
    return rx.cap(1);
}

GoogleImageStruct GooglePicture::ajax_fillStruct(QString s)
{
    GoogleImageStruct tmp;
    tmp.width=              getRegExp("\"width\":\"(.*)\",", s);
    tmp.height=             getRegExp("\"height\":\"(.*)\",", s);
    tmp.descr= tmp.width + " x " + tmp.height;
    tmp.imageId=            getRegExp("\"imageId\":\"(.*)\",", s);
    tmp.tbWidth=            getRegExp("\"tbWidth\":\"(.*)\",", s);
    tmp.tbHeight=           getRegExp("\"tbHeight\":\"(.*)\",", s);
    tmp.unescapedUrl=       getRegExp("\"unescapedUrl\":\"(.*)\",", s);
    tmp.url=                getRegExp("\"url\":\"(.*)\",", s);
    tmp.visibleUrl=         getRegExp("\"visibleUrl\":\"(.*)\",", s);
    tmp.title=              getRegExp("\"title\":\"(.*)\",", s);
    tmp.titleNoFormatting=  getRegExp("\"titleNoFormatting\":\"(.*)\",", s);
    tmp.originalContextUrl= getRegExp("\"originalContextUrl\":\"(.*)\",", s);
    tmp.content=            getRegExp("\"content\":\"(.*)\",", s);
    tmp.contentNoFormatting=getRegExp("\"contentNoFormatting\":\"(.*)\",", s);
    tmp.tbUrl=              getRegExp("\"tbUrl\":\"(.*)\"", s);
    tmp.content = trUtf8(tmp.content.toAscii());
    tmp.contentNoFormatting = trUtf8(tmp.contentNoFormatting.toAscii());
    tmp.gotWithSearchText   = currentSearchString;
    return tmp;
}

QString GooglePicture::ajax_unescapeCharacters(QString s)
{
    QRegExp rx;
    QChar c;
    int pos;
    bool ok;

    rx.setMinimal(true);
    rx.setPattern("\\\\u([0-9|a-f]...)");
    pos=0;
    while ((pos = rx.indexIn(s, pos)) != -1)
    {
        c=rx.cap(1).toInt(&ok, 16);
        s=s.replace(rx.cap(0), c);
    }
    return s;
}

int GooglePicture::ajax_parseResult(QString s, int max_results)
{
    QRegExp rx;
    QString str;
    int pos;

    // replace ascii exadecimal characters (ex: /u003d is '=' )
    s=ajax_unescapeCharacters(s);

    estimatedResultCount=getRegExp("\"estimatedResultCount\":\"(.*)\"", s).toLong();

    rx.setMinimal(true);
    rx.setPattern("results\":\\[(.*)\\],");
    pos=0;
    if ( (pos=rx.indexIn(s,0))==-1) {
        qDebug() << "fillStruct in GooglePicture, didn't found any results";
        return 0;
    }
    str=rx.cap(1);

    rx.setPattern("\\{(.*)\\},");
    pos=0;
    int i=0;
    while ((pos = rx.indexIn(str, pos)) != -1)
    {
        if (tot_found+i>=max_results) break;
        if (pos >= 0 ) { images.append(ajax_fillStruct(rx.cap(1))); i++; }
        pos += rx.matchedLength();
    }
    return i;
}

////////////////////////////////////////////////////////////////
//// FUNCTIONS TO PARSE PUBLIC SEARCH RESULTS //////////////////
////////////////////////////////////////////////////////////////
QString GooglePicture::public_unescapeCharacters(QString s)
{
    QRegExp rx;
    QChar c;
    int pos;
    bool ok;

    rx.setMinimal(true);
    rx.setPattern("\\\\x([0-9|a-f].)");
    pos=0;
    while ((pos = rx.indexIn(s, pos)) != -1)
    {
        c=rx.cap(1).toInt(&ok, 16);
        s=s.replace(rx.cap(0), c);
    }
    return s;
}

int GooglePicture::public_parseResult(QString s, int max_results)
{
    GoogleImageStruct *img;
    QRegExp rx_data,rx_other,rx_start,rx_href,rx_dim;
    int pos,pos2,i=0,nresults;
    QString href_google_thumb_download, conv;

    // To parse result count in different system default languages. Here some examples:
//    <div id=resultStats>About 202,000,000 results<nobr>  (0.08 seconds)&nbsp;</nobr></div>
//    <div id=resultStats>Aproximadamente 186,000,000 resultados<nobr>  (0.23 segundos)&nbsp;</nobr></div>
//    <div id=resultStats> езультатов: примерно 201&nbsp;000&nbsp;000<nobr>  (0,14 сек.)&nbsp;</nobr></div>
//    <div id=resultStats>Приблизна кількість результатів: 192,000,000<nobr>  (0.30 сек.)&nbsp;</nobr></div>
//    <div id=resultStats>Environ 189&nbsp;000&nbsp;000 résultats<nobr>  (0,25 secondes)&nbsp;</nobr></div></div>

//    qDebug() << s;
//    estimatedResultCount=getRegExp("About ([0-9|,]{0,}) results", s).replace(",","").toLong();
    estimatedResultCount=getRegExp("<div id=resultStats>.+ ([\\d|.|,| |&nbsp;]{1,})[ |<].+\\(", s).
            replace(".","").
            replace(",","").
            replace("&nbsp;","").toLong();

//    rx_start.setPattern("setResults[(][[]");
    rx_start.setPattern("setResults[(][[].*[]][)];"); // find the html part which contains links

//    rx_data.setPattern("[[]\".*[]],");
    rx_data.setPattern("/imgres\?.*[]],[[]");  // find single image data
    rx_data.setMinimal(true);

    rx_other.setPattern(",[\"](.*)[\"]");  // find values of image data
    rx_other.setMinimal(true);

    pos = rx_start.indexIn(s, 0);
    if (pos==-1) { qDebug() << "Something changed in the google image page html source. /nThis search criteria must be rewritten"; return 0;}
    s=rx_start.cap(0); // s now contains only links to images
    nresults=0;
    pos=0;
    QString data;
    while ((pos = rx_data.indexIn(s, pos)) != -1 && tot_found+nresults<max_results) {
        img = new GoogleImageStruct;
        data = QByteArray::fromPercentEncoding( public_unescapeCharacters( rx_data.cap(0) ).toAscii() );

//        qDebug() << data;

        // Search for the 1st and 2nd occurrence of rx_href, it's the originalContextUrl and url
        rx_href.setPattern("imgurl=(http://.*)&imgrefurl=(http://.*)&usg.*\",");
        rx_href.setMinimal(true);
        rx_href.indexIn(data,0);
        img->originalContextUrl=public_unescapeCharacters(rx_href.cap(2)).toAscii();
        img->url=public_unescapeCharacters(rx_href.cap(1)).toAscii();

        // Example to combine data for thumbnail (img->tburl):
        // http://tbn1.google.com/images?q=tbn:jKItZeo94DSqzM:http://www.scott-eaton.com/news/uploaded_images/ecorcheZbrush-723429.jpg
        // cap 13 + cap1 + cap 2
        //
        // cap 2 = ID image at google download image thumb
        // cap 18 = source thumbnail href
        // cap 4 = thumb x res
        // cap 5 = thumb y res
        // cap 6 = escaped content (image description)
        // cap 9 = source image values ( with x height x Kb )
        // cap 10 = kind of image (jpg, png ecc)
        // cap 11 = source image site
        // cap 14 = google href for thumbnails (not complete)
        // cap 25 = similar
        // cap 26 = more sizes ??

        pos2 = 0;
        i = 1;
        QString cap;

//        qDebug() << data;
        while ((pos2 = rx_other.indexIn( data, pos2)) != -1) {
            cap = public_unescapeCharacters(rx_other.cap(1));
//            qDebug() << i << " -- " << cap;
            switch (i) {
            case 2: img->imageId=rx_other.cap(1);
                break;
            case 4: img->tbWidth=rx_other.cap(1); break;
            case 5: img->tbHeight=rx_other.cap(1); break;
            case 6:
                conv = trUtf8(cap.toAscii());
                img->content=conv;
                img->contentNoFormatting=conv.remove(QRegExp("<b>")).remove(QRegExp("</b>")); // remove bold html;
                break;
            case 9:
                img->descr=cap.replace("&times;","x");
                rx_dim.setPattern("(.*) &times;");
                rx_dim.indexIn(img->descr,0);
                img->width=rx_dim.cap(1);
                rx_dim.setPattern("&times; (.*) -");
                rx_dim.indexIn(img->descr,0);
                img->height=rx_dim.cap(1);
                rx_dim.setPattern(" - (.*$)");
                rx_dim.indexIn(img->descr,0);
                img->size=rx_dim.cap(1);
                break;
            case 10: img->kind=cap;
                break;
            case 11:
                img->visibleUrl=QUrl(img->unescapedUrl).host();
                break;
            case 14: href_google_thumb_download = cap;
                break;
            case 18: img->unescapedUrl= cap;
                img->tbUrl=QUrl( img->unescapedUrl).fromPercentEncoding(img->unescapedUrl.toLocal8Bit() );
                break;
            }
            pos2 += rx_other.matchedLength();
            i++;
        }
//        img->tbUrl = href_google_thumb_download+"?q=tbn:"+img->imageId+img->url;
        nresults++;
        img->gotWithSearchText = currentSearchString;
        images.append(*img);
        pos += rx_data.matchedLength();

    }
    qDebug() << "Number of image parsed " << i;
    return nresults;
}
