/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef GOOGLEPICTURE_H
#define GOOGLEPICTURE_H

#include <QObject>
#include <QString>
#include <QRegExp>
#include <QList>
#include "network.h"

typedef struct {
    QString width;
    QString height;
    QString imageId;
    QString tbWidth;
    QString tbHeight;
    QString unescapedUrl;
    QString url;
    QString visibleUrl;
    QString title;
    QString titleNoFormatting;
    QString originalContextUrl;
    QString content;
    QString contentNoFormatting;
    QString tbUrl;
    QString descr; // Only filled in public search
    QString size; // Only filled in public search
    QString kind;
    QString gotWithSearchText;
} GoogleImageStruct;


class GooglePicture : public QObject {
    Q_OBJECT
public:
    enum parse_method
    {
        METHOD_AJAX,
        METHOD_PUBLIC
    };

    enum googleSearh {
        SEARCH_OFF,
        SEARCH_news=1,
        SEARCH_face,
        SEARCH_photo,
        SEARCH_clipart,
        SEARCH_lineart,

        SEARCH_medium=1,
        SEARCH_large,
        SEARCH_icon,
        SEARCH_qsvga,
        SEARCH_vga,
        SEARCH_svga,
        SEARCH_xga,
        SEARCH_2mp,
        SEARCH_4mp,
        SEARCH_6mp,
        SEARCH_8mp,
        SEARCH_10mp,
        SEARCH_12mp,
        SEARCH_15mp,
        SEARCH_20mp,
        SEARCH_40mp,
        SEARCH_70mp,

        SEARCH_gray=1,
        SEARCH_color,

        SEARCH_filterOff=0,
        SEARCH_moderate,
        SEARCH_strict,

        SEARCH_reuse=1,
        SEARCH_commercialReuse,
        SEARCH_reuseWithModification,
        SEARCH_commercialReuseWithModification
    };
    static const char *content_type[];
    static const char *image_size[];
    static const char *coloration[];
    static const char *safe_filter[];
    static const char *content_rights[];

    GooglePicture(parse_method method, QObject *parent = 0);
    ~GooglePicture();
    void set_parse_method(parse_method method);
    int  get_images_list(QString search_string,
                         int start_image_number=0, int images_to_get=20,
                         QString content_type="",
                         QString image_size="",
                         QString coloration="",
                         QString safeFilter="",
                         QString rights="",
                         QString site_search=""
                         );
    QStringList getImagesUrl();
    void getThumbsImages();

    QList<GoogleImageStruct> images;
    ulong estimatedResultCount;


private:
    GoogleImageStruct ajax_fillStruct(QString s);
    int ajax_parseResult(QString s, int max_results);
    QString ajax_unescapeCharacters(QString s);
    QString getRegExp(QString pattern, QString str);
    QString public_unescapeCharacters(QString s);
    int public_parseResult(QString s, int max_results);
    network *net_thumbs, *net_search;
    parse_method parse;
    int tot_found;
    QString currentSearchString;

public slots:
    void gotImageSlot(const int ID, const QByteArray &data);
    void queueCompleted();

signals:
    void gotGoogleRecords(QList<GoogleImageStruct> list);
    void gotImageSignal(const QByteArray &data, const GoogleImageStruct &image);
    void googleThumbsGetCompleted();
    void outMessage(const QString &message, int row);
};

#endif // GOOGLEPICTURE_H
