/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QDebug>
#include <QPainter>
#include <QGraphicsSceneContextMenuEvent>
#include <QImage>
#include <QRgb>
#include "imagecontainer.h"

QColor imageContainer::images_color        = QColor(50,50,127);
QColor imageContainer::videos_color        = QColor(127,50,50);
QColor imageContainer::images_text_color   = QColor(255,255,0);
QColor imageContainer::videos_text_color   = QColor(255,255,0);

// Initializer for the label pixmap container for zoomed images
imageContainer::imageContainer(QRect r, QPixmap &pixmap)
{
    Type=this->LABEL;
    l.setGeometry(r);
    l.setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    l.setPixmap(pixmap.scaled(r.size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
    // If I dont do this 2 time (on linux, not on windows), the gradient background
    // is noisy, with blocks taken randomly in the background screen. Maybe a Qt bug? Possibly my graphics card
    // of some debian (with kernel 2.6.32 and nvidia 190.53) opengl library
    l.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(38, 38, 194, 150), stop:0.506726 rgba(0, 0, 0, 80), stop:1 rgba(38, 38, 194, 150));");
    p=QPixmap::grabWidget(&l);
    this->setPixmap(p);
    l.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(38, 38, 194, 150), stop:0.506726 rgba(0, 0, 0, 80), stop:1 rgba(38, 38, 194, 150));");
    p=QPixmap::grabWidget(&l);
    this->setPixmap(p);
}
// Initializer for reflex images
imageContainer::imageContainer(QPixmap pix)
{
    Type=this->REFLEX;

    this->setPixmap(pix);
    make_reflex();
}

imageContainer::imageContainer(const QByteArray &data, int w, int h, imageStruct img)
{
    if (img.kind & CONTAINER_KIND_IMAGE) Type=this->IMAGE;
    if (img.kind & CONTAINER_KIND_VIDEO) Type=this->VIDEO;
    image=img;
    diapo_width=w;
    diapo_heigth=h;
    if (!p.loadFromData(data)) p.load(":/icons/icons/mediadownloader.png");
    make_diapo();
    setupMenu();
    this->setFlags(QGraphicsItem::ItemIsSelectable); // without this flag, on 4.5.3, graphicsItems are not clickable
    this->setAcceptHoverEvents(true);
}

imageContainer::imageContainer(QWidget *w)
{
    Type=this->WIDGET;
    widget=w;
}

imageContainer::~imageContainer()
{
}

void imageContainer::change_label_pixmap(QPixmap &pixmap)
{
    QLabel tmp;
    tmp.setGeometry(l.geometry());
    tmp.setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    tmp.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(38, 38, 194, 150), stop:0.506726 rgba(0, 0, 0, 80), stop:1 rgba(38, 38, 194, 150));");
    tmp.setPixmap(pixmap.scaled(l.size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
    tmp.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(38, 38, 194, 150), stop:0.506726 rgba(0, 0, 0, 80), stop:1 rgba(38, 38, 194, 150));");

    p=QPixmap::grabWidget(&tmp);
    this->setPixmap(p);
    tmp.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(38, 38, 194, 150), stop:0.506726 rgba(0, 0, 0, 80), stop:1 rgba(38, 38, 194, 150));");
    p=QPixmap::grabWidget(&tmp);
    this->setPixmap(p);
    tmp.setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(38, 38, 194, 150), stop:0.506726 rgba(0, 0, 0, 80), stop:1 rgba(38, 38, 194, 150));");
}
///////////////////////////////////////////////////////////
// Reimplemented functions ////////////////////////////////
void imageContainer::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event )
{
    if (event->button() & Qt::LeftButton)
        emit double_clicked(this);
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void imageContainer::resize_diapo(QSize size)
{
    diapo_width=size.width();
    diapo_heigth=size.height();
    make_diapo();
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
//void imageContainer::make_diapo()
//{
//    QPixmap res(diapo_width,diapo_heigth);
//    QPainter painter;

//    p=p.scaled(diapo_width-20, diapo_heigth-80, Qt::KeepAspectRatio, Qt::SmoothTransformation);

//    res.fill(QColor(0,0,0,0));

//    painter.begin(&res);
//    painter.setRenderHint(QPainter::Antialiasing,true);
//    painter.setRenderHint(QPainter::TextAntialiasing,true);
//    painter.setRenderHint(QPainter::HighQualityAntialiasing,true);
//    int x,y, x1,y1, r,g,b;
//    // draw external bevel
//    x=y=0;
//    x1=res.width();
//    y1=res.height();
//    r=g=b=0;
//    if (image.kind & CONTAINER_KIND_IMAGE) b=100;
//    if (image.kind & CONTAINER_KIND_VIDEO) r=100;

//    QPen pen;
//    pen.setWidth(2);
//    for (int i=0; i<6; i++)
//    {
//        r+=20; g+=20; b+=20;
//        pen.setColor(QColor(r,g,b));
//        painter.setPen(pen);
//        painter.drawRoundedRect(x,y, x1,y1, 10,10);
//        x++; y++; x1-=2; y1-=2;
//    }
//    for (int i=0; i<6; i++)
//    {
//        painter.drawRoundedRect(x,y, x1,y1, 10,10);
//        x++; y++; x1-=2; y1-=2;
//    }
//    painter.fillRect(x,y, x1,y1, QColor(r,g,b));

//    // draw image bevel
//    x=(res.width()-p.width())/2;
//    y=10;
//    x1=p.width();
//    y1=p.height();
//    for (int i=0; i<4; i++)
//    {
//        pen.setColor(QColor(r,g,b));
//        painter.setPen(pen);
//        painter.drawRect(x,y, x1,y1);
//        x--; y--; x1+=2; y1+=2;
//        r-=20; g-=20; b-=20;
//    }

//    painter.drawPixmap((res.width()-p.width())/2,10, p.width(),p.height(), p);
//    // Draw text below image
//    pen.setColor(QColor(255,255,0));
//    painter.setPen(pen);
//    painter.setFont(QFont("Helvetica [Cronyx]", 10, 80));
//    painter.drawText(8,8, 40,15, Qt::AlignLeft, QString::number(image.ID+1) );
//    painter.setFont(QFont("Helvetica [Cronyx]", 8, 30));
//    painter.drawText(6, p.height()+17, res.width()-12, res.height()-p.height()-20,
//                     Qt::AlignLeft | Qt::AlignBottom | Qt::TextWordWrap,
//                     image.description_row1+"\n"+
//                     image.description_row2+"\n"+
//                     image.description_row3);
//    painter.end();
//    this->setPixmap(res);
//}
void imageContainer::make_diapo()
{
    QPixmap res(diapo_width,diapo_heigth);
    QPainter painter;
    QString upperText, lowerText;

    QPen pen;
    QBrush *brush;
    QRadialGradient gradient;
    QColor text_color;

    // On mobile draw a bigger image and few text
#ifdef MOBILE_APP
    p=p.scaled(diapo_width-20, diapo_heigth-65, Qt::KeepAspectRatio, Qt::SmoothTransformation);
#else
    p=p.scaled(diapo_width-20, diapo_heigth-80, Qt::KeepAspectRatio, Qt::SmoothTransformation);
#endif

    res.fill(QColor(0,0,0,0));

    painter.begin(&res);
    painter.setRenderHint(QPainter::Antialiasing,true);
    painter.setRenderHint(QPainter::TextAntialiasing,true);
    painter.setRenderHint(QPainter::HighQualityAntialiasing,true);
    int x,y, x1,y1, r=0,g=0,b=0,r2,g2,b2;

    // set parameters
    x=y=0;
    x1=res.width();
    y1=res.height();
    if (image.kind & CONTAINER_KIND_IMAGE) {
        upperText = QString::number(image.ID+1) + " - " + image.description_row1.replace(" x ","x");
#ifdef MOBILE_APP
        lowerText = image.description_row2;
#else
        lowerText = image.description_row2 + "\n" + image.description_row3;
#endif
        r=images_color.red();
        g=images_color.green();
        b=images_color.blue();
        text_color = images_text_color;
    }
    if (image.kind & CONTAINER_KIND_VIDEO) {
        upperText = QString::number(image.ID+1) + " - " + image.description_row3;
#ifdef MOBILE_APP
        lowerText = image.description_row1;
#else
        lowerText = image.description_row1 + "\n" + image.description_row2;
#endif
        r=videos_color.red();
        g=videos_color.green();
        b=videos_color.blue();
        text_color = videos_text_color;
    }

    gradient.setCenter(0,0);
    gradient.setRadius( (res.width()>res.height() ? res.width() : res.height()) );

    r2 = r/2;
    g2 = g/2;
    b2 = b/2;
    gradient.setColorAt(0,      QColor(r, g, b));
    gradient.setColorAt(0.25,   QColor(r2, g2, b2));
    gradient.setColorAt(0.50,   QColor(r, g, b));
    gradient.setColorAt(0.75,   QColor(r2, g2, b2));
    gradient.setColorAt(1,      QColor(r, g, b));

    brush = new QBrush(gradient);
    brush->setStyle(Qt::RadialGradientPattern);
    pen.setBrush(*brush);
    pen.setWidth(2);

//    painter.fillRect(x,y, x1,y1, QColor(r+30,g+30,b+30));
//    painter.fillRect(x+3,y+3, x1-6,y1-6, QColor(r,g,b));
    painter.fillRect(x,y, x1,y1, QColor(r2,g2,b2));
    painter.fillRect(x+3,y+3, x1-6,y1-6, *brush);

    // draw image bevel
    x=(res.width()-p.width())/2;
    y=24;
    x1=p.width();
    y1=p.height();
    for (int i=0; i<4; i++)
    {
        pen.setColor(QColor(r,g,b));
        painter.setPen(pen);
        painter.drawRect(x,y, x1,y1);
        x--; y--; x1+=2; y1+=2;
        r -= r2/4;
        g -= g2/4;
        b -= b2/4;
        if (r>255) r=255;
        if (r<0)   r=0;
        if (g>255) g=255;
        if (g<0)   g=0;
        if (b>255) b=255;
        if (b<0)   b=0;
    }
    painter.drawPixmap((res.width()-p.width())/2, 24, p.width(),p.height(), p);

    // Draw text below image
    int fontSize;
#ifdef Q_WS_S60
    fontSize = 5;
#elif defined(Q_WS_MAEMO_5)
    fontSize = 11;
#else
    fontSize = 9;
#endif
    pen.setColor(text_color);
    painter.setPen(pen);
    painter.setFont(QFont("Helvetica [Cronyx]", fontSize, 80));
    painter.drawText(4,2, res.width()-10,18, Qt::AlignLeft, upperText );

    painter.setFont(QFont("Helvetica [Cronyx]", fontSize, 30));
    painter.drawText(5, p.height()+30, res.width()-10, res.height()-p.height()-33,
                     Qt::AlignLeft | Qt::AlignBottom | Qt::TextWordWrap,
                     lowerText);

    painter.end();
    this->setPixmap(res);
    delete brush;
}


void imageContainer::make_reflex()
{
    // Making snapshot of the pixmap (using QImage to mirror it)
    QImage tmp(this->pixmap().toImage());
    tmp=tmp.convertToFormat(QImage::Format_ARGB32);
    tmp=tmp.mirrored(false,true);
    // Make simple, fast and inaccurate blur effect
    QImage blurred(tmp.width(), tmp.height(), QImage::Format_ARGB32);
    QRgb argb;
    int xx,yy;
    int width,height;
    unsigned int r=0,g=0,b=0,a=0;

    height=tmp.height();
    width=tmp.width();
    for (yy=0; yy<tmp.height(); yy++)
        for (xx=0; xx<width; xx++)
        {
        if (yy<3 || yy>height-3 || xx<3 || xx>width-3) {
            argb=0x00000000;
        }
        else
        {
            argb=tmp.pixel(xx-2,yy);
            r=(uchar)(argb>>16)+1; g=(uchar)(argb>>8); b=(uchar)argb;
            argb=tmp.pixel(xx,yy);
            r+=(uchar)(argb>>16); g+=(uchar)(argb>>8); b+=(uchar)argb;
            argb=tmp.pixel(xx+2,yy);
            r+=(uchar)(argb>>16); g+=(uchar)(argb>>8); b+=(uchar)argb;
            argb=tmp.pixel(xx,yy-2);
            r+=(uchar)(argb>>16); g+=(uchar)(argb>>8); b+=(uchar)argb;
            argb=tmp.pixel(xx,yy+2);
            r+=(uchar)(argb>>16); g+=(uchar)(argb>>8); b+=(uchar)argb;

            r/=5; g/=5; b/=5;
        }
        argb=(r<<16) + (g<<8) + b;
        blurred.setPixel(xx,yy, argb);
    }
    // Make transparency
    for (yy=0; yy<height; yy++)
        for (xx=0; xx<width; xx++)
        {
        a=(110*(height-yy))/height+2;// 110 = starting transparency value (255=opaque)
        argb=(a<<24) + blurred.pixel(xx,yy);
        blurred.setPixel(xx,yy, argb);
    }

// Set a gradiend of transparency to paint over the image
//    QLinearGradient gradient(0,0,0, blurred.height());
//    gradient.setColorAt(0, QColor(0,0,0, 160));
//    gradient.setColorAt(0.8, QColor(0,0,0, 255));
//    QPainter p(&blurred);
//    p.fillRect(0, 0, blurred.width(), blurred.height(), gradient);
//    p.end();

    // Add the pixmap to the scene
    QPixmap tmp2=QPixmap::fromImage(blurred);

    this->setPixmap(tmp2);

    // With QGraphicsBlurEffect the graphisView slow down a lot, so I implement it directly to the image above
//#if QT_VERSION > 0x040600
//            QGraphicsBlurEffect *effect = new QGraphicsBlurEffect;
//            effect->setBlurRadius(5);
//            reflex->setGraphicsEffect(effect);
//            // no more then 1 effects :(
//            //        QGraphicsOpacityEffect *effect2 = new QGraphicsOpacityEffect;
//            //        effect2->setOpacity(0.5);
//            //        reflex->setGraphicsEffect(effect2);
//#endif
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void imageContainer::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    // If ctrl pressed we should send itemClicked event
//    if (event->modifiers() & Qt::ControlModifier) return;

    selectedAction = menu.exec(event->screenPos());
}

void imageContainer::setupMenu()
{
    removeAction = menu.addAction(tr("remove"));
    if (image.kind & CONTAINER_KIND_IMAGE)
    {
        setAsBackgroundAction = menu.addAction(tr("set as background"));
        connect(setAsBackgroundAction, SIGNAL(triggered()), this, SLOT(on_setAsBackground_clicked()) );
    }
    if (image.kind==CONTAINER_KIND_IMAGE ||
        image.kind==CONTAINER_KIND_VIDEO ) {
        downloadAction = menu.addAction(tr("download"));
#ifndef MOBILE_APP
        browseAction = menu.addAction(tr("browse"));
        connect(browseAction, SIGNAL(triggered()), this, SLOT(on_browse_clicked()) );
#endif
        connect(downloadAction, SIGNAL(triggered()), this, SLOT(on_download_clicked()) );
    }
    if (image.kind==CONTAINER_KIND_VIDEO_OTHER ) {
        downloadAction = menu.addAction(tr("download"));
        connect(downloadAction, SIGNAL(triggered()), this, SLOT(on_download_clicked()) );
    }
#ifndef MOBILE_APP
    if (image.kind & CONTAINER_KIND_VIDEO) {
        playWithMplayer = menu.addAction(tr("play with mplayer"));
        connect(playWithMplayer, SIGNAL(triggered()), this, SLOT(on_playWithMplayer_clicked()) );
    }
#endif
    if (image.kind & CONTAINER_KIND_VIDEO) {
        extractAudio = menu.addAction(tr("extract audio"));
        connect(extractAudio, SIGNAL(triggered()), this, SLOT(on_extractAudio_clicked()) );
    }

    connect(removeAction, SIGNAL(triggered()), this, SLOT(on_remove_clicked()) );

    menu.setStyleSheet(
            "QMenu { "
            "background-color: rgba(0, 50, 180, 210); "
            "color: rgb(255, 255, 0); "
            "border-width: 2px; "
            "border-style: outset; "
            "border-radius: 5px; "
            "border-color: rgb(255, 255, 255); "
            "} "
            "QMenu::item:selected { "
            "background-color: rgba(0, 0, 255,255); "
            "}"
            );
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void imageContainer::on_extractAudio_clicked()
{
    emit on_menu_extractAudio_clicked(this);
}

void imageContainer::on_playWithMplayer_clicked()
{
    emit on_menu_playWithMplayer_clicked(this);
}

void imageContainer::on_remove_clicked()
{
    emit on_menu_remove_clicked(this);
}

void imageContainer::on_download_clicked()
{
    QString download_url;
    if (image.kind==CONTAINER_KIND_IMAGE)
        download_url=image.image_url;
    if (image.kind==CONTAINER_KIND_VIDEO)
        download_url=( youTube::getTubeVideoInfo(image.youTubeInfo)==0 ? "" :
                       image.youTubeInfo.video_info.at(0).url_retrieve);

    emit on_menu_download_clicked(this);
}

void imageContainer::on_browse_clicked()
{
    emit on_menu_browse_clicked(image.image_source);
}

void imageContainer::on_setAsBackground_clicked()
{
    emit on_menu_setAsBackground_clicked(this);
}
