/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <math.h>
#include "mainwindow.h"
#include "imageviewer2d.h"
#include "imagecontainer.h"
#include "network.h"
#include "save.h"
#include "defines.h"

#include <QDebug>
#include <QFormLayout>
#include <QGroupBox>
#include <QPushButton>
#include <QAbstractScrollArea>
#include <QLinearGradient>
#include <QScrollBar>
#include <QPolygon>
#include <QProcess>
#include <QGesture>

network *ImageViewer2d::net_sync=0;

ImageViewer2d::ImageViewer2d(QWidget *parent)
{
    Q_UNUSED(parent)
#ifdef QT_OPENGL_SUPPORT
//    qgl=new QGLWidget(new QGLContext(QGLFormat(QGL::Rgba | QGL::SampleBuffers)));
    qgl=new QGLWidget(new QGLContext(QGLFormat(QGL::Rgba | QGL::DoubleBuffer | QGL::AlphaChannel)));// | QGL::HasOverlay)));
    this->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);

    this->setViewport(qgl);

//    setAttribute(Qt::WA_PaintOnScreen);
//    setAttribute(Qt::WA_NoSystemBackground);
//    setAutoFillBackground(true);
#endif
    this->setRenderHint(QPainter::Antialiasing);

    this->setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing);

//    this->setCacheMode(QGraphicsView::CacheBackground);
#ifndef QT_OPENGL_SUPPORT
    // This setStyle is for non openGL widget
    this->viewport()->setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0.216, x2:0, y2:0.676, stop:0.00444444 rgba(95, 95, 203, 255), stop:0.391111 rgba(48, 48, 179, 255), stop:1 rgba(0, 0, 0, 255));");
#endif

    connect(&timer, SIGNAL(timeout()), this, SLOT(inertia()));

    n_widget=0;

    contextMenu=true;

    this->setScene(&sceneOGL);
    zoom_factor=1;
    tot_DY=0;
    tot_DX=0;
    grid_rows=2;

    this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setTransformationAnchor(QGraphicsView::NoAnchor);

//    connect(this, SIGNAL(itemDragging(imageContainer*)) ,
//            this, SLOT(onItemDragging(imageContainer*)) );

#ifdef MOBILE_APP
    rotationSensor = new QRotationSensor(this);
    rotationSensor->start();
    if (!rotationSensor->isActive()) {
        qWarning("RotationSensor didn't start!");
    }
    else {
        connect(rotationSensor, SIGNAL(readingChanged()), SLOT(updateRotation()));
        rotReading = rotationSensor->reading();
    }
    // Handle tap and hold gesture
    this->grabGesture(Qt::TapAndHoldGesture);
#endif
}

ImageViewer2d::~ImageViewer2d()
{
    QGraphicsItem *item;
    QGraphicsProxyWidget *cast;

#ifdef MOBILE_APP
    delete rotationSensor;
#endif

    foreach (item , sceneOGL.items())
    {
        cast=qgraphicsitem_cast<QGraphicsProxyWidget*>(item);
        sceneOGL.removeItem(item);
        delete cast;
    }

    itemsList.clear();
#ifdef QT_OPENGL_SUPPORT
    delete qgl;
#endif
}
////////////////////////////////////////////
////////////////////////////////////////////
#ifdef MOBILE_APP
bool ImageViewer2d::event ( QEvent * event )
{
    if (event->type() == QEvent::Gesture)
        return gestureEvent(static_cast<QGestureEvent*>(event));
    return QWidget::event(event);
}

bool ImageViewer2d::gestureEvent(QGestureEvent *event)
{
    if (QGesture *swipe = event->gesture(Qt::SwipeGesture))
        swipeTriggered(static_cast<QSwipeGesture *>(swipe));
    else if (QGesture *pan = event->gesture(Qt::PanGesture))
        panTriggered(static_cast<QPanGesture *>(pan));
    if (QGesture *pinch = event->gesture(Qt::PinchGesture))
        pinchTriggered(static_cast<QPinchGesture *>(pinch));
    if (QGesture *tapAndHold = event->gesture(Qt::TapAndHoldGesture))
        tapAndHoldTriggered(static_cast<QTapAndHoldGesture *>(tapAndHold));
    return true;
}

void ImageViewer2d::swipeTriggered(QSwipeGesture *gesture)
 {
     if (gesture->state() == Qt::GestureFinished) {
//         if (gesture->horizontalDirection() == QSwipeGesture::Left
//             || gesture->verticalDirection() == QSwipeGesture::Up)
//             goPrevImage();
//         else
//             goNextImage();
//         update();
     }
 }

void ImageViewer2d::tapAndHoldTriggered(QTapAndHoldGesture *gesture)
 {
     if (gesture->state() == Qt::GestureFinished) {
         if (contextMenu)
             QGraphicsView::contextMenuEvent(new QContextMenuEvent(QContextMenuEvent::Other,
                                                                   gesture->position().toPoint()));
     }
 }

void ImageViewer2d::pinchTriggered(QPinchGesture *gesture)
 {
     if (gesture->state() == Qt::GestureFinished) {

     }
 }

void ImageViewer2d::panTriggered(QPanGesture *gesture)
 {
     if (gesture->state() == Qt::GestureFinished) {

     }
 }

void ImageViewer2d::focusInEvent ( QFocusEvent * event )
{
    Q_UNUSED(event)
//    qDebug() << "ImageViewer2d::focusInEvent";
    rotationSensor->start();
}

void ImageViewer2d::focusOutEvent ( QFocusEvent * event )
{
    Q_UNUSED(event)
//    qDebug() << "ImageViewer2d::focusOutEvent";
    rotationSensor->stop();
}
#endif // MOBILE_APP
////////////////////////////////////////////
////////////////////////////////////////////
void ImageViewer2d::resizeItems(QSize size)
{
    imageContainer *item;

    setImageContainerSize(size, QSize(widget_H_separation,widget_V_separation));
    foreach (item, itemsList)
        item->resize_diapo(size);
    this->arrange_items(false);
}

void ImageViewer2d::setImageContainerSize(QSize ic_size, QSize separation)
{
    widget_width=ic_size.width();
    widget_height=ic_size.height();
    widget_H_separation=separation.width();
    widget_V_separation=separation.height();
}

////////////////////////////////////////////
////////////////////////////////////////////
QStringList ImageViewer2d::getImageUrls() // return mapped urls with a default file extension
{
    QStringList list;
    imageContainer *item;

    list.clear();
    foreach (item , itemsList)
    {
        if (item->Type==imageContainer::VIDEO)
            if (youTube::getTubeVideoInfo(item->image.youTubeInfo)>0)
                list.append(item->image.youTubeInfo.video_info.at(0).url_retrieve);

        if (item->Type==imageContainer::IMAGE)
            list.append(item->image.image_url);
    }
    return list;
}
////////////////////////////////////////////
////////////////////////////////////////////
void ImageViewer2d::add_widget(imageContainer *w)
{
    sceneOGL.addItem(w);
    itemsList.append(w);
    n_widget++;
    arrange_items(true);
}

// Items here are not added to graphicsView. Only QList<imageContainer *> itemsList is filled
imageContainer* ImageViewer2d::add_imageContainer(const QByteArray &data, imageStruct image)
{
    // Add item to the scene
    imageContainer *w= new imageContainer(data, widget_width, widget_height, image);
    itemsList.append(w);
    n_widget++;

    connect(w, SIGNAL(on_menu_browse_clicked(QString)), this, SLOT(onItem_Browse_clicked(QString)) );
    connect(w, SIGNAL(on_menu_extractAudio_clicked(imageContainer*)),
            this, SLOT(onItem_extractAudio_clicked(imageContainer*)) );
    connect(w, SIGNAL(on_menu_download_clicked(imageContainer*)),
            this, SLOT(onItem_Download_clicked(imageContainer*)) );
    connect(w, SIGNAL(on_menu_remove_clicked(imageContainer*)), this, SLOT(onRemoveWidget(imageContainer*)) );
    connect(w, SIGNAL(on_menu_setAsBackground_clicked(imageContainer*)), this, SLOT(onItem_SetAsBackground_clicked(imageContainer*)) );
    connect(w, SIGNAL(on_menu_playWithMplayer_clicked(imageContainer*)), this, SLOT(onItem_playWithMplayer_clicked(imageContainer*)) );
    connect(w, SIGNAL(double_clicked(imageContainer*)),
            this, SLOT(onItemDoubleClicked(imageContainer*)) );
    return w;
}

// add items but sort before. While downloading could happens that one item is downloaded
// before his preceding item, so display those after all are downloaded
void ImageViewer2d::addItemList_to_scene()
{
    // Bubblesort itemsList by itemsList.ID
    int a,b;
    a=0;
    while (a < itemsList.size()-1)
    {
        b=a+1;
        while (b < itemsList.size())
        {
            if (itemsList.at(a)->image.ID > itemsList.at(b)->image.ID)
                itemsList.swap(b,a);
            b++;
        }
        a++;
    }
    for (int i=start_nwidget; i<itemsList.size(); i++) {


        // Make preset for item animation
#if QT_VERSION > 0x040600
        effect = new QGraphicsOpacityEffect;
        effect->setOpacity(0);

        fade = new QPropertyAnimation(effect, "opacity");

        fade->setDuration(700);
        fade->setStartValue(-0.5);
        fade->setEndValue(1);

        timeLine = new QTimeLine(500);
        timeLine->setFrameRange(0, 60);

        animation = new QGraphicsItemAnimation;
        animation->setTimeLine(timeLine);

        for (int n = 0; n <= 60; ++n) {
            animation->setScaleAt(n / 60.0, 1, n / 60.0);
            animation->setShearAt(n / 60.0, 0, n / 60.0-1);
        }

        itemsList[i]->setGraphicsEffect(effect);
        animation->setItem(itemsList[i]);

        connect(fade, SIGNAL(finished()), fade, SLOT(deleteLater()));
        connect(timeLine, SIGNAL(finished()), timeLine, SLOT(deleteLater()));
        connect(timeLine, SIGNAL(finished()), animation, SLOT(deleteLater()));

        // Start item animation at same randome time
        int r = rand()%1000;
        QTimer::singleShot(r, fade, SLOT(start()) );
        QTimer::singleShot(r, timeLine, SLOT(start()) );
#endif
        sceneOGL.addItem(itemsList.at(i));
    }
    arrange_items(false);
}

// Get a list of all displayed items
QList<imageStruct> ImageViewer2d::getImageStructList()
{
    QList<imageStruct> ret;

    foreach (imageContainer *i, itemsList) {
        ret.append(i->image);
    }
    return ret;
}

void ImageViewer2d::add_all_images(QList<imageStruct> imgs)
{
    int i;

    start_nwidget=itemsList.size();
    images=imgs;
    net_thumbs = new network(false, MainWindow::pointerToMainWindow);
    connect(net_thumbs, SIGNAL(downloadFinished(int,QByteArray)),
            this, SLOT(onDownloaded(int,QByteArray)) );
    connect(net_thumbs, SIGNAL(queue_completed()),
            this, SLOT(onDownloadThumbsCompleted()));

    i=0;
    while (i<images.size())
    {
        net_thumbs->get(images.at(i).image_tburl);
        i++;
    }
}

void ImageViewer2d::onDownloaded(int id,const QByteArray &data)
{
    images[id].ID=id+start_nwidget;
    add_imageContainer(data, images.at(id));
}

void ImageViewer2d::onDownloadThumbsCompleted()
{
    addItemList_to_scene();
    net_thumbs->deleteLater();
    emit networking_finished();
}

void ImageViewer2d::onItem_extractAudio_clicked(imageContainer *item)
{
    emit item_extractAudio_button_clicked(item);
}

void ImageViewer2d::onItem_Download_clicked(imageContainer *item)
{
    emit item_download_button_clicked(item);
}

void ImageViewer2d::onItem_playWithMplayer_clicked(imageContainer *item)
{
    emit item_playWithMplayer_button_clicked(item);
}

void ImageViewer2d::onItem_SetAsBackground_clicked(imageContainer *item)
{
    emit item_setAsBackground_button_clicked(item);
}

void ImageViewer2d::onItem_Browse_clicked(const QString &url)
{
    emit item_browse_button_clicked(url);
}

void ImageViewer2d::onRemoveWidget(imageContainer *w)
{
    int i;

    i=itemsList.indexOf(w);
    if (i!=-1) {
        sceneOGL.removeItem(w);
        n_widget--;
        itemsList.removeAt(i);
        arrange_items(false);
    }
}

// If last==true, arrange only latest item
void ImageViewer2d::arrange_items(bool last)
{
    int x,y,i;
    qreal scale;

    emit this->outMessage(tr("Items listed: ") + QString::number(itemsList.size()),0);
    if (last) i=itemsList.size()-1;
    else {
        i=0;
        // Remove all reflex images
        QGraphicsItem * item;
        foreach (item , sceneOGL.items())
        {
            if (item->type()==item->UserType+1)
            {
                sceneOGL.removeItem(item);
                delete item;
            }
        }
    }

    while (i<itemsList.size())
    {
        if ( ((qreal)widget_width/itemsList[i]->pixmap().width()) < ((qreal)widget_height/itemsList[i]->pixmap().height()) )
            scale=((qreal)widget_width/itemsList[i]->pixmap().width());
        else scale=((qreal)widget_height/itemsList[i]->pixmap().height());
        if (last)
            itemsList[i]->scale(scale, scale);

        x=(i/grid_rows) * (widget_width+widget_H_separation);
        y=(i%grid_rows) * (widget_height+widget_V_separation);
        itemsList[i]->setPos(x,y);

        itemsList[i]->setCacheMode(QGraphicsItem::ItemCoordinateCache); //DeviceCoordinateCache

        if (i%grid_rows==grid_rows-1) { // we are in the bottom row, so make reflex effect
            imageContainer *reflex= new imageContainer(itemsList[i]->pixmap());
            reflex->scale(scale, scale);
            y+=widget_height;//+widget_V_separation+5;
            reflex->setPos(x,y);
            reflex->setCacheMode(QGraphicsItem::ItemCoordinateCache);
            sceneOGL.addItem(reflex);
        }
        i++;
    }
    sceneOGL.setSceneRect(sceneOGL.itemsBoundingRect());
//    this->setSceneRect(sceneOGL.itemsBoundingRect());
}

void ImageViewer2d::set_rows(int n)
{
    if (n<=0) n=1;
    grid_rows=n;
    this->arrange_items(false);
}

int ImageViewer2d::get_rows()
{
    return grid_rows;
}

void ImageViewer2d::add_row()
{
    grid_rows++;
    this->arrange_items(false);
}

void ImageViewer2d::remove_row()
{
    if (grid_rows-1>=1)
    {
        grid_rows--;
        this->arrange_items(false);
    }
}

////////////////////////////////////////////
void ImageViewer2d::clear_scene()
{
    QGraphicsItem * item;
    foreach (item , sceneOGL.items()) {
        sceneOGL.removeItem(item);
        delete item;
    }

    itemsList.clear();
    sceneOGL.setSceneRect(sceneOGL.itemsBoundingRect());
    this->setSceneRect(0,0, 0,0);
    this->resetCachedContent();
    this->resetTransform();
    this->resetMatrix();
    n_widget=0;
}
////////////////////////////////////////////
// ZOOMING
////////////////////////////////////////////
void ImageViewer2d::zoomItem(imageContainer *from_item, imageContainer *zoom_item)
{
    QPointF start_pos;
    QPointF end_pos;
    qreal start_scale,end_scale;

    start_scale=from_item->p.width()/this->width();
    end_scale=this->mapToScene(0,0, this->width(),this->height()).boundingRect().width()/this->width();

    start_pos=QPointF(from_item->x() , from_item->y());
    end_pos=this->mapToScene(0,0);

    if (zoom_item->type()==imageContainer::LABEL)
        sceneOGL.addItem(zoom_item);
    if (zoom_item->type()==imageContainer::VIDEO)
        sceneOGL.addWidget(zoom_item->widget);
#if QT_VERSION > 0x040600
    zoom_item->setPos(start_pos);
#else
    zoom_item->setPos(end_pos);
#endif
    zoom_item->setCacheMode(QGraphicsItem::ItemCoordinateCache);
    zoom_item->setZValue(100);

    // Set animation (fade, translation, scale)
#if QT_VERSION > 0x040600
    if (zoom_item->type()==imageContainer::VIDEO)
    {
//        resizeVideoContainer(dynamic_cast<mediaPlayer*>(zoom_item->widget)->curr_video_size);
    }

    if (zoom_item->type()==imageContainer::LABEL)
    {
        timeLine = new QTimeLine(2000);
        timeLine->setFrameRange(0, 100);
        timeLine->setEasingCurve(QEasingCurve(QEasingCurve::OutCubic));

        animation = new QGraphicsItemAnimation;
        animation->setItem(zoom_item);
        animation->setTimeLine(timeLine);

        animation->setPosAt(0, start_pos);
        animation->setPosAt(1, end_pos );

        animation->setScaleAt(0, start_scale, start_scale);
        animation->setScaleAt(1, end_scale,end_scale);
        timeLine->start();
    }

    // Set fade animation
    QGraphicsOpacityEffect *effect = new QGraphicsOpacityEffect;
    effect->setOpacity(0);
    zoom_item->setGraphicsEffect(effect);

    QPropertyAnimation *fade = new QPropertyAnimation(effect, "opacity");
    fade->setDuration(1000);
    fade->setStartValue(0);
    fade->setEndValue(1);

    fade->start(QAbstractAnimation::DeleteWhenStopped);
#endif
}

void ImageViewer2d::onMediaItemClosed()
{
    media->deleteLater();
    if (media->kind==CONTAINER_KIND_IMAGE && net_sync!=0) net_sync->reply_synchronous->abort();
}


void ImageViewer2d::onItemDoubleClicked(imageContainer *item)
{
    if (item->type()==item->REFLEX) return; // If doubleclicked on a reflex image exit

    emit item_double_clicked();

#ifdef Q_WS_PM
    QString command;
    if (item->image.youTubeInfo.video_info.size()==0)
        if (youTube::getTubeVideoInfo(item->image.youTubeInfo)==0) return;
    MbCookieJar cookies;
    cookies.copyFrom(network::cookieJar);
    if (!cookies.writeCookies(QDir::homePath()+"/.mediadownloader/MDcookies.txt")) return;
    command +=  "smplayer \"" + item->image.youTubeInfo.video_info.at(item->image.youTubeInfo.ID_preferred_video_format).url_play +"\"";
    if (item->image.kind==CONTAINER_KIND_VIDEO_OTHER || item->image.kind==CONTAINER_KIND_VIDEO_LOCAL)
        command = "smplayer \"" + item->image.image_url + "\"";
    qDebug() << "Running: " << command;
    QProcess::startDetached ( command );
    return;
#endif

    if (item->image.kind==CONTAINER_KIND_VIDEO)
    {
#ifdef PHONON_LIB
#ifndef MOBILE_APP
            // Inside QGraphicsView, video colors are misplaced with Qt <= 4.7.0 (linux only)
            // ref: http://bugreports.qt.nokia.com/browse/QTBUG-8737
            if (item->image.youTubeInfo.video_info.size()==0)
                if (youTube::getTubeVideoInfo(item->image.youTubeInfo)==0) return;
    #ifndef MOBILE_APP
            media = new mediaPlayer(false, CONTAINER_KIND_VIDEO, MainWindow::pointerToMainWindow);
    #else
            media = new mediaPlayer(false, CONTAINER_KIND_VIDEO);
    #endif
            connect(media, SIGNAL(destroyed()), this, SLOT(onMediaItemClosed()) );
    //        media->show();
            media->goNormalScreen();
            media->play(item->image.youTubeInfo);

            return;
        }
        if (item->image.kind==CONTAINER_KIND_VIDEO_LOCAL ||
            item->image.kind==CONTAINER_KIND_VIDEO_OTHER )
        {
            media = new mediaPlayer(false, item->image.kind, MainWindow::pointerToMainWindow);
            connect(media, SIGNAL(destroyed()), this, SLOT(onMediaItemClosed()) );
            media->show();
            media->goNormalScreen();
            media->play(item->image.image_url);
#endif

#ifdef MOBILE_APP  // Use QtMultimediaKit instead of phonon
        if (item->image.youTubeInfo.video_info.size()==0)
            if (youTube::getTubeVideoInfo(item->image.youTubeInfo)==0) return;
        videoPlayerMobile *mediaPlayer = new videoPlayerMobile;
        mediaPlayer->play(item->image.youTubeInfo);
        mediaPlayer->show();
#endif  // MOBILE_APP
#endif // PHONON_LIB
        return;
    }

    if (item->image.kind & CONTAINER_KIND_IMAGE)
    {
        media = new mediaPlayer(true, item->image.kind, MainWindow::pointerToMainWindow);
        connect(media, SIGNAL(destroyed()), this, SLOT(onMediaItemClosed()) );
//        connect(media->pixContainer, SIGNAL(doubleClicked()), this, SLOT(onMediaItemClosed()) );


        media->show();
        media->goNormalScreen();
        media->play(item->p);

        // Download original image
        QByteArray b;
        QPixmap s;

        net_sync=new network(false, MainWindow::pointerToMainWindow);
        connect(media, SIGNAL(stateChanged()), net_sync, SLOT(onSetProgressWidgetOnTop()) );
        connect(media, SIGNAL(destroyed()), net_sync, SLOT(onGetSynchronousCancelled()) );
        b=net_sync->get_synchronous(item->image.image_url, true, media->geometry());
        if (!b.isEmpty())
        {
            s.loadFromData(b);
            media->play(s);
            media->show();
        }
        net_sync->deleteLater();
        net_sync=0;
    }
}
////////////////////////////////////////////
////////////////////////////////////////////
////////////////////////////////////////////
// Enable/Disable menu
void ImageViewer2d::contextMenuEvent ( QContextMenuEvent * event )
{
    if (contextMenu)
        QGraphicsView::contextMenuEvent(event);
}
void ImageViewer2d::enableContextMenuEvent ( )
{
    contextMenu=true;
}
void ImageViewer2d::disableContextMenuEvent ( )
{
    contextMenu=false;
}

////////////////////////////////////////////
////////////////////////////////////////////
////////////////////////////////////////////
// Drag event
//void ImageViewer2d::keyPressEvent ( QKeyEvent * event )
//{
//    if (event->key() & Qt::Key_Control) ctrl_pressed=true;
//}
//void ImageViewer2d::keyReleaseEvent ( QKeyEvent * event )
//{
//    if (event->key() & Qt::Key_Control) ctrl_pressed=false;
//}

// Maybe I'll do drag&drop later
//void ImageViewer2d::onItemDragging(imageContainer *item)
//{
//    qDebug() << ctrl_pressed;
//    QDrag *drag=new QDrag(this);
//    QMimeData mimeData;
//    QList<QUrl> l;
//    QByteArray b;
//
//    l.append(item->image.url);
////    mimeData.setText(item->image.descr);
////    mimeData.setData("text/plain", b);
//    mimeData.setUrls(l);
////    mimeData.setData("x-uuencode", b);
//    drag->setMimeData(&mimeData);
//    // Download the big image
//    Qt::DropAction dropAction = drag->exec();
//
////    if (dropAction==2)
////    {
////        net_sync=new network();
////        net_sync->setProgressbar(true,true,14,200,0,12);
////        net_sync->setProgressbarVisible(true);
////        b=net_sync->get_synchronous(item->image.url);
////        if (!b.isEmpty())
////        {
//////            mimeData->setData("x-uuencode", b);
//////            mimeData->setImageData(QImage(b.data()));
//////            drag->setMimeData(mimeData);
//////            drag->setPixmap(item->p);
////        }
////        net_sync->deleteLater();
////        net_sync=0;
////    }
//    qDebug() << "Drop" << dropAction;
//}

////////////////////////////////////////////
////////////////////////////////////////////
////////////////////////////////////////////
// Mouse Events for scene dynamic
void ImageViewer2d::mousePressEvent ( QMouseEvent *e )
{
    mouse_buttons = e->buttons();
//    if (ctrl_pressed && (e->button() & Qt::LeftButton)) // We start dragging if over an item
//    {
//        if (QGraphicsItem *i=sceneOGL.itemAt(this->mapToScene(e->x(),e->y())))
//            if ( dynamic_cast<imageContainer*>(i)->type()==dynamic_cast<imageContainer*>(i)->ITEM )
//            emit itemDragging(dynamic_cast<imageContainer*>(i));
//        return;
//    }

    if (e->button() & Qt::LeftButton)
    {
        if (timer.isActive())
            timer.stop();
        inertia_X=0;
        inertia_Y=0;
        DX = start_X = e->x();
        DY = start_Y = e->y();
#ifdef MOBILE_APP
        rotationSensor->stop();
#endif
    }
    QGraphicsView::mousePressEvent(e);
}

void ImageViewer2d::mouseReleaseEvent ( QMouseEvent *e )
{
    mouse_buttons = e->buttons();
    if (e->button() & Qt::LeftButton)
    {

        DX += e->x();
        DY += e->y();
        tot_DX = (e->x() - start_X)/20.0;
        tot_DY = (e->y() - start_Y)/20.0;
        if (!timer.isActive()) timer.start(20);
#ifdef MOBILE_APP
        rotationSensor->start();
#endif
    }
    QGraphicsView::mouseReleaseEvent(e);
}

void ImageViewer2d::mouseMoveEvent ( QMouseEvent *e )
{
    if (mouse_buttons & Qt::LeftButton)
    {
        tot_DX=e->x()-DX;
        tot_DY=e->y()-DY;
        this->translate(tot_DX,tot_DY);
        DX=e->x();
        DY=e->y();
        checkLimitReached();
    }
    QGraphicsView::mouseMoveEvent(e);
}

void ImageViewer2d::inertia()
{
    if (n_widget==0) return;
    QRectF r=this->sceneRect();
    QPolygon p=this->mapFromScene(r);
    static QPolygon prec=0;
    prec=p;

    tot_DX+=inertia_X;
    tot_DY+=inertia_Y;
    inertia_X=0;
    inertia_Y=0;
    // if inertia is close to 0, stop it
    if ( tot_DX<=1.0e-1 && tot_DX>=-1.0e-1 &&
         tot_DY<=1.0e-1 && tot_DY>=-1.0e-1 ) {
        timer.stop();
        tot_DX=0;
        tot_DY=0;
    }
//    if (mouse_buttons & Qt::LeftButton) return;
    tot_DX/=1.03;
    tot_DY/=1.03;
    this->translate(tot_DX,tot_DY);

    p=this->mapFromScene(r);
    if (p.point(0).x()==prec.point(0).x()) {
        // If right edge reached, emit the signal and resize view, else block horizontal scrolling
        if (!checkLimitReached())
            tot_DX=0;
    }
    if (p.point(0).y()==prec.point(0).y()) tot_DY=0;
}

void ImageViewer2d::stopInertia()
{
    tot_DX=0;
    tot_DY=0;
}

#ifdef MOBILE_APP
void ImageViewer2d::updateRotation()
{
    qreal rx,ry;
    ry=0;
    // Check orientation
    if (this->width() > this->height())
        rx=rotReading->y();
    else
        rx=rotReading->x() * -1;

    // If the angle of rotation is small, start inertia
    if (rx>-10 && rx<10)
    {
        rx=0;
        if ( abs(tot_DX)>0 && !timer.isActive()) {
            timer.start(20);
        }
        return;
    }

    rx /= 50.0;

    // If the user invert the rotation, stop inerzia
    if (tot_DX>0 && rx<0) tot_DX=0;
    if (tot_DX<0 && rx>0) tot_DX=0;

    tot_DX += rx;
    tot_DY += ry;
    this->translate(tot_DX,tot_DY);
    checkLimitReached();
}
#endif

bool ImageViewer2d::checkLimitReached()
{
    QRectF r=this->sceneRect();
    QPolygon p=this->mapFromScene(r);

    // If right edge reached, emit the signal and resize view, else block horizontal scrolling
    if (/*tot_DX==0 && */mapToScene(this->width(), this->height()).x() >= r.width()-1) {

        emit onScrollRightLimitReached();
        return true;
    }
    return false;
}

// Expand width by a 5*itemWidgetWidth
void ImageViewer2d::expandWidth()
{
    QRectF r=this->sceneRect();
    QPolygonF itemRect = mapToScene(0, 0,
                                    widget_width+widget_H_separation,
                                    widget_height+widget_V_separation);
    QRectF newSceneRect(0,0,
                        r.width()+itemRect.boundingRect().width()*5,
                        r.height());
//    this->setSceneRect(newSceneRect);
    sceneOGL.setSceneRect(newSceneRect);
}

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
void ImageViewer2d::setBackgroundImage(QImage image)
{
    backgroundImage = image;
    sceneOGL.update();
}
void ImageViewer2d::setBackgroundGradient(QColor top, QColor mid, QColor bottom)
{
    QRectF rect;
    rect=this->mapToScene(0,0, this->width(), this->height()).boundingRect();

    lg.setStart(rect.topLeft());
    lg.setFinalStop(rect.bottomLeft());

    top_color=top;
    mid_color=mid;
    bottom_color=bottom;
    lg.setColorAt(0.0, top_color);
    lg.setColorAt(0.5, mid_color);
    lg.setColorAt(1.0, bottom_color);
    sceneOGL.update();
}

void ImageViewer2d::drawBackground(QPainter *painter,
                              const QRectF &f)
{
    Q_UNUSED(f);

    QRectF rect;
    rect=this->mapToScene(0,0, this->width(), this->height()).boundingRect();
    lg.setStart(rect.topLeft());
    lg.setFinalStop(rect.bottomLeft());

    painter->drawImage(rect, backgroundImage);
    painter->fillRect( rect, lg );
}



//#include <GL/gl.h>
//void ImageViewer2d::drawBackground(QPainter *painter,
//                              const QRectF &)
//{
//#if QT_VERSION > 0x040600
//    if ( !((painter->paintEngine()->type() != QPaintEngine::OpenGL) ||
//         (painter->paintEngine()->type() != QPaintEngine::OpenGL2))
//        ) {
//        qWarning("OpenGLScene: drawBackground needs a "
//                 "QGLWidget to be set as viewport on the "
//                 "graphics view");
//        return;
//    }
//#endif
//    // draw a rectagle with color vertex in the background
//    glMatrixMode (GL_MODELVIEW);
//    glPushMatrix ();
//    glLoadIdentity ();
//    glMatrixMode (GL_PROJECTION);
////    glRotatef(ImageViewer2d::tot_DX*10.0, 0.0,1.0,0.0);
//    glPushMatrix ();
//    glLoadIdentity ();
//    glBegin (GL_QUADS);
//        glColor3f(0.0f, 0.0f, 0.0f);    glVertex3i (-1, -1, -1);
//        glColor3f(0.0f, 0.0f, 0.0f);    glVertex3i (1,  -1, -1);
//        glColor3f(0.0f, 0.0f, 0.5f);    glVertex3i (1,   1, -1);
//        glColor3f(0.0f, 0.0f, 0.5f);    glVertex3i (-1,  1, -1);
//    glEnd ();
//    glPopMatrix ();
//    glMatrixMode (GL_MODELVIEW);
//    glPopMatrix ();
////    glMatrixMode (GL_MODELVIEW);
////    glPushMatrix ();
////    glLoadIdentity ();
////    glMatrixMode (GL_PROJECTION);
////    glRotatef(ImageViewer2d::tot_DX*10.0, 0.0,1.0,0.0);
////    glMatrixMode (GL_MODELVIEW);
////    glPopMatrix ();
////    glMatrixMode (GL_MODELVIEW);
////    glPushMatrix ();
////    glLoadIdentity ();
////    glMatrixMode (GL_PROJECTION);
////    glRotatef(ImageViewer2d::tot_DX*10.0, 0.0,1.0,0.0);
////    glMatrixMode (GL_MODELVIEW);
////    glPopMatrix ();
//}
