/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include "imagewidget.h"
#include "ui_imagewidget.h"
#include "customizeimagewidget.h"
#include "mainwindow.h"
#include "settings.h"
#include <QToolTip>
#include <QMessageBox>

imageWidget::imageWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::imageWidget)
{
    settings conf;

    ui->setupUi(this);
    scene = new ImageViewer2d(this);
    ui->horizontalLayout->addWidget(scene);

    // Restore settings
    scene->set_rows(conf.conf->value("graphicsView/rows",2).toInt());

    scene->setImageContainerSize( QSize(conf.conf->value("graphicsView/thumb_width",150).toInt(),
                                        conf.conf->value("graphicsView/thumb_height",200).toInt()),
                                  QSize(conf.conf->value("graphicsView/thumb_h_separation",15).toInt(),
                                        conf.conf->value("graphicsView/thumb_v_separation",15).toInt()));
    scene->setBackgroundGradient(
                                    QColor(conf.conf->value("graphicsView/top_colorR",53).toInt(),
                                           conf.conf->value("graphicsView/top_colorG",60).toInt(),
                                           conf.conf->value("graphicsView/top_colorB",180).toInt(),
                                           conf.conf->value("graphicsView/top_colorA",200).toInt()
                                           ),
                                    QColor(conf.conf->value("graphicsView/mid_colorR",0).toInt(),
                                           conf.conf->value("graphicsView/mid_colorG",0).toInt(),
                                           conf.conf->value("graphicsView/mid_colorB",50).toInt(),
                                           conf.conf->value("graphicsView/mid_colorA",220).toInt()
                                           ),
                                    QColor(conf.conf->value("graphicsView/bottom_colorR",220).toInt(),
                                           conf.conf->value("graphicsView/bottom_colorG",220).toInt(),
                                           conf.conf->value("graphicsView/bottom_colorB",110).toInt(),
                                           conf.conf->value("graphicsView/bottom_colorA",200).toInt()
                                           )
                                    );
    setSlidersValue(QSize(conf.conf->value("graphicsView/thumb_width",150).toInt(),
                          conf.conf->value("graphicsView/thumb_height",200).toInt() ));

    // When release slider, resize thumbnails
    connect(ui->horizontalSlider, SIGNAL(sliderReleased()), this, SLOT(onSliderChanged()) );
    connect(ui->verticalSlider, SIGNAL(sliderReleased()), this, SLOT(onSliderChanged()) );

    // While moving sliders, show value
    connect(ui->horizontalSlider, SIGNAL(sliderMoved(int)), this, SLOT(onHorizontalSliderMovement(int)) );
    connect(ui->verticalSlider, SIGNAL(sliderMoved(int)), this, SLOT(onVerticalSliderMovement(int)) );


    customizeDialog = new customizeImageWidget(this);
    connect(customizeDialog, SIGNAL(top_color_changed(QColor)),
            this, SLOT(onTop_color_changed(QColor)) );
    connect(customizeDialog, SIGNAL(mid_color_changed(QColor)),
            this, SLOT(onMid_color_changed(QColor)) );
    connect(customizeDialog, SIGNAL(bottom_color_changed(QColor)),
            this, SLOT(onBottom_color_changed(QColor)) );
    connect(customizeDialog, SIGNAL(images_color_changed(QColor)),
            this, SLOT(onImages_color_changed(QColor)) );
    connect(customizeDialog, SIGNAL(videos_color_changed(QColor)),
            this, SLOT(onVideos_color_changed(QColor)) );
    connect(customizeDialog, SIGNAL(images_text_color_changed(QColor)),
            this, SLOT(onImages_text_color_changed(QColor)) );
    connect(customizeDialog, SIGNAL(videos_text_color_changed(QColor)),
            this, SLOT(onVideos_text_color_changed(QColor)) );

    connect(customizeDialog, SIGNAL(image_choosen(bool,QString)),
            this, SLOT(onImage_choosen(bool,QString)) );

    imageContainer::images_color = scene->images_color  = QColor(conf.conf->value("graphicsView/images_colorR",35).toInt(),
                                                                 conf.conf->value("graphicsView/images_colorG",70).toInt(),
                                                                 conf.conf->value("graphicsView/images_colorB",190).toInt()
                                                                 );
    imageContainer::videos_color = scene->videos_color  = QColor(conf.conf->value("graphicsView/videos_colorR",127).toInt(),
                                                                 conf.conf->value("graphicsView/videos_colorG",4).toInt(),
                                                                 conf.conf->value("graphicsView/videos_colorB",6).toInt()
                                                                 );
    imageContainer::images_text_color  = scene->images_text_color  = QColor(conf.conf->value("graphicsView/images_text_colorR",255).toInt(),
                                                                            conf.conf->value("graphicsView/images_text_colorG",255).toInt(),
                                                                            conf.conf->value("graphicsView/images_text_colorB",0).toInt()
                                                                            );
    imageContainer::videos_text_color  = scene->videos_text_color  = QColor(conf.conf->value("graphicsView/videos_text_colorR",255).toInt(),
                                                                            conf.conf->value("graphicsView/videos_text_colorG",255).toInt(),
                                                                            conf.conf->value("graphicsView/videos_text_colorB",0).toInt()
                                                                            );

    customizeDialog->initDialog(scene->top_color,
                                scene->mid_color,
                                scene->bottom_color,
                                scene->images_color,
                                scene->videos_color,
                                scene->images_text_color,
                                scene->videos_text_color
                                );
}

imageWidget::~imageWidget()
{
    settings conf;
    conf.store_settings("graphicsView/top_colorR", scene->top_color.red());
    conf.store_settings("graphicsView/top_colorG", scene->top_color.green());
    conf.store_settings("graphicsView/top_colorB", scene->top_color.blue());
    conf.store_settings("graphicsView/top_colorA", scene->top_color.alpha());
    conf.store_settings("graphicsView/mid_colorR", scene->mid_color.red());
    conf.store_settings("graphicsView/mid_colorG", scene->mid_color.green());
    conf.store_settings("graphicsView/mid_colorB", scene->mid_color.blue());
    conf.store_settings("graphicsView/mid_colorA", scene->mid_color.alpha());
    conf.store_settings("graphicsView/bottom_colorR", scene->bottom_color.red());
    conf.store_settings("graphicsView/bottom_colorG", scene->bottom_color.green());
    conf.store_settings("graphicsView/bottom_colorB", scene->bottom_color.blue());
    conf.store_settings("graphicsView/bottom_colorA", scene->bottom_color.alpha());
    conf.store_settings("graphicsView/rows", scene->get_rows());
    conf.store_settings("graphicsView/thumb_width", scene->widget_width);
    conf.store_settings("graphicsView/thumb_height", scene->widget_height);
    conf.store_settings("graphicsView/thumb_H_separation", scene->widget_H_separation);
    conf.store_settings("graphicsView/thumb_V_separation", scene->widget_V_separation);

    conf.store_settings("graphicsView/images_colorR", scene->images_color.red());
    conf.store_settings("graphicsView/images_colorG", scene->images_color.green());
    conf.store_settings("graphicsView/images_colorB", scene->images_color.blue());
    conf.store_settings("graphicsView/videos_colorR", scene->videos_color.red());
    conf.store_settings("graphicsView/videos_colorG", scene->videos_color.green());
    conf.store_settings("graphicsView/videos_colorB", scene->videos_color.blue());
    conf.store_settings("graphicsView/images_text_colorR", scene->images_text_color.red());
    conf.store_settings("graphicsView/images_text_colorG", scene->images_text_color.green());
    conf.store_settings("graphicsView/images_text_colorB", scene->images_text_color.blue());
    conf.store_settings("graphicsView/videos_text_colorR", scene->videos_text_color.red());
    conf.store_settings("graphicsView/videos_text_colorG", scene->videos_text_color.green());
    conf.store_settings("graphicsView/videos_text_colorB", scene->videos_text_color.blue());

    delete customizeDialog;
    delete scene;
    delete ui;
}

void imageWidget::onSliderChanged()
{
    scene->resizeItems(QSize(ui->horizontalSlider->value(), ui->verticalSlider->value()));
}

void imageWidget::onHorizontalSliderMovement(int i)
{
    Q_UNUSED(i);
    QToolTip::showText(MainWindow::pointerToMainWindow->pos()+ui->horizontalSlider->pos()+QPoint(10,230),
                       "Width: "+QString::number(ui->horizontalSlider->value()) , this);
}

void imageWidget::onVerticalSliderMovement(int i)
{
    Q_UNUSED(i);
    QToolTip::showText(MainWindow::pointerToMainWindow->pos()+ui->horizontalSlider->pos()+QPoint(10,230),
                       "Height: "+QString::number(ui->verticalSlider->value()) , this);
}

void imageWidget::setSlidersValue(QSize size)
{
    ui->horizontalSlider->setValue(size.width());
    ui->verticalSlider->setValue(size.height());
}

//////////////////////////////
//////////////////////////////
// Customize backgroud widget
void imageWidget::on_PB_customize_clicked()
{
    customizeDialog->show();
}

void imageWidget::onTop_color_changed(QColor c)
{
    scene->top_color=c;
    scene->setBackgroundGradient(scene->top_color,scene->mid_color,scene->bottom_color);
}

void imageWidget::onMid_color_changed(QColor c)
{
    scene->mid_color=c;
    scene->setBackgroundGradient(scene->top_color,scene->mid_color,scene->bottom_color);
}

void imageWidget::onBottom_color_changed(QColor c)
{
    scene->bottom_color=c;
    scene->setBackgroundGradient(scene->top_color,scene->mid_color,scene->bottom_color);
}

void imageWidget::onImage_choosen(bool checked, QString fileName)
{
    if (fileName.isEmpty() && checked) {
        scene->setBackgroundImage(QImage(":/icons/icons/warp.jpg"));
        return;
    }
    if (!checked) {
        scene->setBackgroundImage(QImage(0,0));
    }
    else {
        QFile f(fileName);
        if (!f.open(QIODevice::ReadOnly))
        {
            QMessageBox::critical(0,QObject::tr("Background image can't be read"),
                                  QObject::tr("Can't open: ")+fileName,
                                  QMessageBox::Ok );
            return;
        }
        QImage img(fileName);
        scene->setBackgroundImage(img);
    }
}

void imageWidget::on_PB_open_download_directory_clicked()
{
    emit PB_open_download_directory_clicked();
}

void imageWidget::onImages_color_changed(QColor c)
{
    scene->images_color=c;
    imageContainer::images_color = c;
    scene->resizeItems(QSize(ui->horizontalSlider->value(), ui->verticalSlider->value()));
}

void imageWidget::onVideos_color_changed(QColor c)
{
    scene->videos_color=c;
    imageContainer::videos_color = c;
    scene->resizeItems(QSize(ui->horizontalSlider->value(), ui->verticalSlider->value()));
}

void imageWidget::onImages_text_color_changed(QColor c)
{
    scene->images_text_color=c;
    imageContainer::images_text_color = c;
    scene->resizeItems(QSize(ui->horizontalSlider->value(), ui->verticalSlider->value()));
}

void imageWidget::onVideos_text_color_changed(QColor c)
{
    scene->videos_text_color=c;
    imageContainer::videos_text_color = c;
    scene->resizeItems(QSize(ui->horizontalSlider->value(), ui->verticalSlider->value()));
}
