/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QByteArray>
#include <QPixmap>
#include <QPicture>
#include <QImage>
#include <QIODevice>
#include <QTranslator>
#include <QFileInfo>
#include <QIntValidator>

#include "googlepicture.h"
#include "youtube.h"
#include "imagewidget.h"
#include "mediaplayer.h"
#include "browse.h"
#include "save.h"
#include "about.h"
#include "styles.h"
#include "ffmpegfrontend.h"
#include "firefoxmonitor.h"

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:

    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    QList<imageStruct> convert_GoogleStruct_to_imageStruct(QList<GoogleImageStruct> google);
    QList<imageStruct> convert_youTube_to_imageStruct(QList<YouTubeStruct> tubes);
    void getVideoFeed(StandardVideoFeeds feed);
    void downloadItemsList(QList<imageStruct> list);

    QPixmap *pix;
    QPicture pic;
    QImage img;
    static QWidget *pointerToMainWindow;

protected:
    void changeEvent(QEvent *e);

private:
    void copy_application_files();
    void fill_CB_style();
    Ui::MainWindow *ui;
    QIntValidator* validator;
    QIntValidator* validator2;
    QTranslator *translator;
    imageWidget *viewer;
    browse *browser;
    about *_about;
    QProcess ffmpegProcess;
    QFileInfo savedFilename;
    ffmpegFrontend *ffmpeg;
    ffmpegOutput *output;
    save *down;
    QFileInfo saverestore_fileName;
    styles *style;

    GooglePicture *google;

    firefoxMonitor *cacheMonitor;
    bool isMonitoringFirefoxCache;
    bool addByScrolling;
    int start_image_number, images_to_get;
    int feed_start_number, feeds_to_get;

private slots:
    void on_SB_max_concurrent_downloads_valueChanged(int i);
    void on_CB_delete_original_file_clicked();
    void onFfmpeg_args_changed();
    void onCB_ffmpeg_preset_changed(int i);
    void on_PB_make_preset_clicked();
    void on_CB_use_ffmpeg_clicked(bool checked);
    void on_PB_multiple_search_from_file_clicked();
    void onSB_feed_start_number_text_changed(QString);
    void on_PB_directory_to_import_clicked();
    void on_PB_start_import_clicked();
    void on_PB_slide_start_clicked();
    void onCB_max_resolution_changed(int i);
    void on_PB_mostRecent_clicked();
    void on_PB_mostPopular_clicked();
    void on_PB_mostViewed_clicked();
    void on_PB_topFavorites_clicked();
    void on_PB_topRated_clicked();
    void on_PB_hide_clicked();
    void onCB_language_changed(int i);
    void onCB_style_changed(int i);
    void on_PB_about_clicked();
    void on_PB_removeRow_clicked();
    void on_PB_addRow_clicked();
    void on_PB_directory_clicked();
    void on_PB_clear_clicked();
    void on_PB_exit_clicked();
    void on_PB_download_clicked();
    void on_PB_search_clicked();
    int  doSearch(QStringList searchString);
    void onDownloadItem(imageContainer *item);
    void onBrowseItem(const QString url);
    void onSetAsBackgroundItem(imageContainer *item);
    void onPlayWithMplayerItem(imageContainer *item);
    void onBrowserButtonClosePressed();
    void onEnableSearchButtonRequest();
    void onEnableDownloadButtonRequest();
    void onOutMessage(const QString &message, int row);
    void on_PB_save_to_new_file_clicked();
    void on_PB_replace_all_to_saved_clicked();
    void on_PB_append_to_existing_file_clicked();
    void on_PB_append_saved_to_current_clicked();
    void on_PB_saverestore_clicked();
    void on_RB_only_mp4_clicked(bool b);
    void on_PB_monitor_clicked();
    void onScrollRightLimitReached();
    void onPB_open_download_directory_clicked();

    void onExtractAudio(imageContainer *item);
    void onStop_ffmpeg_clicked();
    void ffmpegStdout();
    void ffmpegStderr();
    void ffmpegFinished( int exitCode );
    void ffmpegStarted();
    void ffmpegError(QProcess::ProcessError err);
};

#endif // MAINWINDOW_H
