/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef MBCOOKIEJAR_H
#define MBCOOKIEJAR_H

#include <QObject>
#include <QNetworkCookieJar>
#include <QDebug>
#include <QFile>
#include <QString>
#include <QDateTime>

///////////////////////////////////////////////////////
// custom cookieJar ///////////////////////////////////
class MbCookieJar : public QNetworkCookieJar
{
    Q_OBJECT
public:
    explicit MbCookieJar(QObject *parent = 0)
        : QNetworkCookieJar(parent)
    {
    }

    void copyFrom(const MbCookieJar *from) {
        if (from!=0) setAllCookies(from->allCookies());
    }

    void appendFrom(const MbCookieJar *from) {
        QList<QNetworkCookie> allFrom = from->allCookies();
        QList<QNetworkCookie> allThis = allCookies();
        for (int i=0; i< allFrom.size(); i++) {
            if (allThis.indexOf(allFrom.at(i)) == -1)
                allThis.append(allFrom.at(i));
        }
        setAllCookies(allThis);
    }

    // for debug purpose
    void displayAllCookies() {
        QNetworkCookie c;
        qDebug() << "*************************";
        foreach( c, allCookies()) {
            qDebug() << c.domain() << c.name() << c.value();
        }
        qDebug() << "*************************";
    }

    QList<QNetworkCookie> cookiesList() {
        return allCookies();
    }

    // write cookies to file in Netscape format
    bool writeCookies(QString url) {
        QNetworkCookie c;
        QFile f(url);
        QString s;

        if (!f.open(QIODevice::WriteOnly)) {
            qDebug() << "Cannot write cookies file";
            return false;
        }
        foreach( c, allCookies()) {
            s .clear();
            s = c.domain() + "\t";
            s += "TRUE\t";
            s += c.path()  + "\t";
            s += ( c.isSecure() ? "TRUE\t" : "FALSE\t" );
            s += QString::number(c.expirationDate().toTime_t())+"\t"; // toMSecsSinceEpoch()/1000) + "\t";
            s += c.name() + "\t";
            s += c.value() + "\t\n";
            f.write(s.toAscii());
        }
        f.close();
        return true;

    }
};

#endif // MBCOOKIEJAR_H
