/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#include <QDebug>
#include <QtCore/qmath.h>
#include "mbslider.h"

#define INNER_COLOR (is_down ? handlerInnerColorPressed : handlerInnerColor)

///////////////////////////////////////////////////
///////////////////////////////////////////////////
// MBSlider Class
///////////////////////////////////////////////////
MBSlider::MBSlider(QWidget *parent) :
    QAbstractSlider(parent)
{
    // buffer minimum and maximum are the same as slider
    setBufferValue(1000);
//    setValue(0);
    bufferedColor = QColor(255,0,0);
    unbufferedColor = QColor(Qt::gray);
    emptyColor    = QColor(150,150,150);
    setHandlerOuterColor(QColor(50,50,255));
    setHandlerInnerColor(QColor(255,0,0));
    setHandlerInnerColorPressed(QColor(0,255,0));
    handlerEdge = ( width() > height() ?
                height()>>1 :
                width()>>1 );
    is_down = false;
    memset(bufferChunks, 0, 1000);
    calcBufferRects();

    setOrientation(Qt::Horizontal);
    setMinimum(0);
    setMaximum(1000);
    setStyleSheet("QSlider::handle:horizontal {"
                          "background: green;"
                          "border: 1px solid #5c5c5c;"
                          "border-radius: 6px;"
                          "}"
                          );
}

MBSlider::~MBSlider()
{
}
///////////////////////////////////////////////////
// Protected functions
///////////////////////////////////////////////////
void MBSlider::mousePressEvent ( QMouseEvent * event )
{
    Q_UNUSED(event)
//    if (event->button() == Qt::LeftButton)
//    {
//        is_down = true;
//        int newPos;
//        if (orientation() == Qt::Vertical) {
//            newPos = minimum() + ((maximum()-minimum()) *
//                                  (height()-event->y())) / height();
////            handler->move( 0, newPos);
//        }
//        else {
//            newPos = minimum() + ((maximum()-minimum()) *
//                                  event->x()) / width();
////            handler->move( newPos, 0);
//        }
//        newPos = 100.0*newPos/(double)(maximum()-minimum());
//        setValue(newPos);
//        event->accept();
//    }
//    QAbstractSlider::mousePressEvent(event);
}

void MBSlider::mouseMoveEvent ( QMouseEvent * event )
{
    if (event->buttons() & Qt::LeftButton)
    {
        is_down = true;
        int newPos;
        if (orientation() == Qt::Vertical) {
            newPos = minimum() + ((maximum()-minimum()) *
                                  (height()-event->y())) / height();
//            handler->move( 0, newPos);
        }
        else {
            newPos = minimum() + ((maximum()-minimum()) *
                                  event->x()) / width();
//            handler->move( newPos, 0);
        }
        newPos = 1000.0*newPos/(double)(maximum()-minimum());
        calcNewValue(newPos);
        //setValue(newPos);
    }
    repaint();
//    QAbstractSlider::mouseMoveEvent(event);
}

void MBSlider::mouseReleaseEvent ( QMouseEvent * event )
{
    if (event->button() == Qt::LeftButton)
    {
        is_down = false;
        int newPos;
        if (orientation() == Qt::Vertical) {
            newPos = minimum() + ((maximum()-minimum()) *
                                  (height()-event->y())) / height();
//            handler->move( 0, newPos);
        }
        else {
            newPos = minimum() + ((maximum()-minimum()) *
                                  event->x()) / width();
//            handler->move( newPos, 0);
        }
        newPos = 1000.0*newPos/(double)(maximum()-minimum());
        emit sliderMoved(newPos);
        calcNewValue(newPos);
        //setValue(newPos);
        repaint();
    }
}

void MBSlider::resizeEvent ( QResizeEvent * event )
{
    Q_UNUSED(event)
    handlerEdge = ( width() > height() ?
                       height()>>1 :
                       width()>>1 );
    calcBufferRects();
}

void MBSlider::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    QPointF center;

    center = QPointF( valuePos, height()>>1 );

    /////////////////////////////////////////////////////
    // Buffer progress
    /////////////////////////////////////////////////////
    // fill background
    painter.fillRect( event->rect(), Qt::transparent);
    // fill buffered area
    gradientL2 = QLinearGradient( QPointF(0,0),
//                                 QPointF(handlerEdge>>3, handlerEdge>>3)
                                 QPointF(0, handlerEdge)
                                 );
    gradientL2.setColorAt(0,    unbufferedColor );
    gradientL2.setColorAt(0.3,  unbufferedColor.lighter(170) );
    gradientL2.setColorAt(0.7,  unbufferedColor.lighter(170) );
    gradientL2.setColorAt(1,    unbufferedColor );
    gradientL2.setSpread(QGradient::ReflectSpread);
    painter.fillRect( bufferRect, gradientL2 );

    gradientL = QLinearGradient( QPointF(0,0),
//                                 QPointF(handlerEdge>>3, handlerEdge>>3)
                                 QPointF(0, handlerEdge)
                                 );
    gradientL.setColorAt(0,    bufferedColor.lighter(170) );
    gradientL.setColorAt(0.3,  bufferedColor );
    gradientL.setColorAt(0.7,  bufferedColor );
    gradientL.setColorAt(1,    bufferedColor.lighter(170) );
    gradientL.setSpread(QGradient::ReflectSpread);

    painter.setBrush(gradientL);
    QRect r;
    double step;
    step = (double)bufferRect.width()/1000.0;
    r.setY(bufferRect.y());
    r.setHeight(bufferRect.height());
    for (int n=0; n < 1000; n++)
        if (bufferChunks[n]) {
//            painter.fillRect( bufferRect, gradientL );
            r.setX( step*n );
            r.setWidth( qCeil(step) );
            painter.fillRect( r, gradientL );
        }


    /////////////////////////////////////////////////////
    // Handler
    /////////////////////////////////////////////////////
//    painter.fillRect( event->rect(), Qt::transparent);

    // Outer circle
    gradient = QRadialGradient( center,
                               handlerEdge>>1,
                               center );
    gradient.setColorAt(0.5,  handlerOuterColor );
    gradient.setColorAt(0.8,  handlerOuterColor.lighter(150) );
    gradient.setColorAt(0.98,  handlerOuterColor );
    gradient.setColorAt(1,    QColor(0,0,0) );

    painter.setBrush(gradient);
    painter.drawEllipse( center,
                        handlerEdge>>1, handlerEdge>>1);

    // Inner circle
    gradient = QRadialGradient( center,
                               handlerEdge>>2,
                               center );
    gradient.setColorAt(0,    Qt::white);
    gradient.setColorAt(0.3,  INNER_COLOR.lighter(150) );
    gradient.setColorAt(0.9,  INNER_COLOR );
    gradient.setColorAt(1,    QColor(0,0,0) );

    painter.setBrush(gradient);
    painter.drawEllipse(QPoint( valuePos, height()>>1 ),
                        handlerEdge>>2, handlerEdge>>2 );


//    painter.drawRect(this->geometry());
//    painter.drawRect(bufferRect);
}
///////////////////////////////////////////////////
// Public functions
///////////////////////////////////////////////////
void MBSlider::setBufferedColor(QColor c)
{
    bufferedColor = c;
}

void MBSlider::setUnbufferedColor(QColor c)
{
    unbufferedColor = c;
}

void MBSlider::setEmptyColor(QColor c)
{
    emptyColor = c;
}

void MBSlider::setHandlerOuterColor(QColor c)
{
    handlerOuterColor = c;
}

void MBSlider::setHandlerInnerColor(QColor c)
{

    handlerInnerColor = c;
}

void MBSlider::setHandlerInnerColorPressed(QColor c)
{

    handlerInnerColorPressed = c;
}

int MBSlider::getValue()
{
    return value;
}

bool MBSlider::getValue(int perc)
{
    return bufferChunks[perc];
}

bool MBSlider::isDown()
{
    return is_down;
}

void MBSlider::clearBufferChunks()
{
    for (int n=0; n<1000; n++)
        bufferChunks[n] = 0;
    repaint();
}

///////////////////////////////////////////////////
// Private functions
///////////////////////////////////////////////////
void MBSlider::calcBufferRects()
{
//    bufferRect = QRect(this->geometry().x(),
//                       ( this->geometry().height() >> 3 )*3 ,
//                       (double)this->geometry().width(),
//                       this->geometry().height() >> 2 );
    bufferRect = QRect(0,
                       ( this->geometry().height() >> 3 )*3 ,
                       (double)this->geometry().width(),
                       ( this->geometry().height() >> 3 )*2 );
}

void MBSlider::calcNewValue(int v)
{
    value = v;
    if (value > maximum()) value = maximum();
    if (value < minimum()) value = minimum();
    valuePos = (double)width()* ( (double)(value-minimum())/(double)(maximum()-minimum()) );
}
///////////////////////////////////////////////////
// Slots
///////////////////////////////////////////////////
void MBSlider::setBufferChunks(int percStart, int percEnd)
{
    bool b=false;
    for (int n=percStart; n<=percEnd; n++) {
        if (bufferChunks[n] == 1) b=true;
        bufferChunks[n] = 1;
    }
    if (b) return;
    repaint();
}

void MBSlider::setBufferValue(int percent)
{
    bufferValue = percent / ((double)maximum()-minimum());
    calcBufferRects();
    repaint();
}

void MBSlider::setValue(int v)
{
    if (is_down) return;
    value = v;
    calcNewValue(v);
    repaint();
    emit valueChanged(value);
}
