/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef MBSLIDER_H
#define MBSLIDER_H

#include <QAbstractSlider>
#include <QAbstractButton>
#include <QMouseEvent>
#include <QPainter>
#include <QColor>

class MBSlider : public QAbstractSlider
{
    Q_OBJECT

public:
    explicit MBSlider(QWidget *parent = 0);
    ~MBSlider();

    void setBufferedColor(QColor c);
    void setUnbufferedColor(QColor c);
    void setEmptyColor(QColor c);
    void setHandlerOuterColor(QColor c);
    void setHandlerInnerColor(QColor c);
    void setHandlerInnerColorPressed(QColor c);
    int getValue();
    bool getValue(int perc);
    bool isDown();
    void clearBufferChunks();

private:
    void calcNewValue(int v);
    void calcBufferRects();
    QAbstractButton *handler;
    double bufferValue;
    QColor bufferedColor;
    QColor unbufferedColor;
    QColor emptyColor;
    QRadialGradient gradient;
    QLinearGradient gradientL;
    QLinearGradient gradientL2;
    QColor handlerOuterColor;
    QColor handlerInnerColor;
    QColor handlerInnerColorPressed;
    int handlerEdge;
    QRect bufferRect;
    int value;
    int valuePos; // defines the position of handler to be painted
    bool is_down; // handler status
    char bufferChunks[1000];

protected:
    void mousePressEvent ( QMouseEvent * event );
    void mouseReleaseEvent ( QMouseEvent * event );
    void mouseMoveEvent ( QMouseEvent * event );
    void paintEvent ( QPaintEvent * event );
    void resizeEvent ( QResizeEvent * event );

public slots:
    void setBufferChunks(int percStart, int percEnd);
    void setBufferValue(int percent);
    void setValue(int v);

signals:
    void valueChanged(int value);

};

#endif // MBSLIDER_H
