/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifdef PHONON_LIB
#include <phonon/backendcapabilities.h>
#endif
#include <QDebug>
#include <QMouseEvent>
#include <QApplication>
#include <QDesktopWidget>
#include <QMessageBox>
#include <QStyle>
#include <QIcon>
#include <QImage>
#include "mediaplayer.h"
#include "popuptext.h"
#include "mainwindow.h"

// delay for updating values with a timer, seems that without there are random crashes
// I like the sliders have tracking, so implemented this timer
#define DELAY_BUG 200

clickableLabel::clickableLabel(QWidget *parent)/* :
    QLabel(parent, Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint)*/
{
    Q_UNUSED(parent)
    this->setFocusPolicy(Qt::StrongFocus);
//    this->setFocus();
//    this->activateWindow();
}

void clickableLabel::mousePressEvent ( QMouseEvent *e ) {
    if (e->button() & Qt::LeftButton)
        emit clicked();
}
void clickableLabel::mouseDoubleClickEvent ( QMouseEvent * e ) {
    if (e->button() & Qt::LeftButton)
        emit doubleClicked();
}

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#ifdef PHONON_LIB
adjustVideoParameters::adjustVideoParameters(Phonon::VideoWidget * vw, QRect r, QWidget *parent) :
        QWidget(parent)
{
    this->setWindowFlags(Qt::WindowStaysOnTopHint);
//    this->setFocusPolicy(Qt::StrongFocus);
//    this->setFocus();
//    this->activateWindow();

    videoWidget = vw;

    this->setGeometry(r.x(), r.y(), r.width(), r.height());

    layout = new QGridLayout(this);
    brightness = new QSlider(this);
    contrast   = new QSlider(this);
    hue        = new QSlider(this);
    saturation = new QSlider(this);

    brightnessLabel = new QLabel(tr("brightness "),this);
    brightnessLabel->setMaximumSize(100,23);
    brightnessLabel->setMinimumSize(100,23);
    brightness->setRange(-100,100);
    brightness->setOrientation(Qt::Horizontal);
    brightness->setMaximumSize(175,15);
    brightness->setMinimumSize(175,15);
    brightness->setValue(vw->brightness()*100);
    connect(brightness, SIGNAL(valueChanged(int)), this, SLOT(onBrightness_moved(int)));

    contrastLabel = new QLabel(tr("contrast "),this);
    contrastLabel->setMaximumSize(100,23);
    contrastLabel->setMinimumSize(100,23);
    contrast->setRange(-100,100);
    contrast->setMaximumSize(175,15);
    contrast->setMinimumSize(175,15);
    contrast->setOrientation(Qt::Horizontal);
    contrast->setValue(vw->contrast()*100);
    connect(contrast, SIGNAL(valueChanged(int)), this, SLOT(onContrast_moved(int)));

    hueLabel = new QLabel(tr("hue "),this);
    hueLabel->setMaximumSize(100,23);
    hueLabel->setMinimumSize(100,23);
    hue->setMaximumSize(175,15);
    hue->setMinimumSize(175,15);
    hue->setRange(-100,100);
    hue->setOrientation(Qt::Horizontal);
    hue->setValue(vw->hue()*100);
    connect(hue, SIGNAL(valueChanged(int)), this, SLOT(onHue_moved(int)));

    saturationLabel = new QLabel(tr("saturation "),this);
    saturationLabel->setMaximumSize(100,23);
    saturationLabel->setMinimumSize(100,23);
    saturation->setMaximumSize(175,15);
    saturation->setMinimumSize(175,15);
    saturation->setRange(-100,100);
    saturation->setOrientation(Qt::Horizontal);
    saturation->setValue(vw->saturation()*100);
    connect(saturation, SIGNAL(valueChanged(int)), this, SLOT(onSaturation_moved(int)));

    PB_reset = new QPushButton(this);
    PB_reset->setMaximumSize(40,23);
    PB_reset->setMinimumSize(40,23);
    PB_reset->setObjectName(QString::fromUtf8("PB_reset"));
    PB_reset->setText("reset");
    connect(PB_reset, SIGNAL(clicked()), this, SLOT(onPB_reset_clicked()) );

    layout->addWidget(brightnessLabel,0,0);
    layout->addWidget(brightness,0,1);
    layout->addWidget(contrastLabel,1,0);
    layout->addWidget(contrast,1,1);
    layout->addWidget(hueLabel,2,0);
    layout->addWidget(hue,2,1);
    layout->addWidget(saturationLabel,3,0);
    layout->addWidget(saturation,3,1);
    layout->addWidget(PB_reset,4,1);

    this->setStyleSheet("QLabel {color: rgb(255,255,0); background-color: rgb(0,0,0);} "
                        "QSlider { margin: 0px; background-color: "
                        "qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, "
                        "stop:0 rgba(205, 0, 0, 255), stop:0.5 rgba(0, 0, 0, 180), "
                        "stop:1 rgba(0, 193, 79, 255)); border-radius: 4px;}");
    canUpdate=true;
}

adjustVideoParameters::~adjustVideoParameters()
{
    delete brightness;
    delete contrast;
    delete hue;
    delete saturation;
    delete brightnessLabel;
    delete contrastLabel;
    delete hueLabel;
    delete saturationLabel;
    delete layout;
}

void adjustVideoParameters::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    painter.fillRect( event->rect(), Qt::black );
    painter.setPen(Qt::yellow);
    painter.drawRect( event->rect().x(), event->rect().y(),
                      event->rect().width()-1, event->rect().height()-1);
}

void adjustVideoParameters::onTimerFinished()
{
    canUpdate=true;
}

void adjustVideoParameters::onPB_reset_clicked()
{
    onBrightness_moved(0);
    onContrast_moved(0);
    onHue_moved(0);
    onSaturation_moved(0);
}

void adjustVideoParameters::onBrightness_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        brightness->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setBrightness(0);
        return;
    }
    videoWidget->setBrightness( (qreal(i)/100.0) );
}

void adjustVideoParameters::onContrast_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        contrast->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setContrast(0);
        return;
    }

    videoWidget->setContrast( (qreal(i)/100.0) );
}

void adjustVideoParameters::onHue_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        hue->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setHue(0);
        return;
    }
    videoWidget->setHue( (qreal(i)/100.0) );
}

void adjustVideoParameters::onSaturation_moved(int i)
{
    if (i>-8 && i<8) {
        i=0;
        saturation->setValue(i);
    }
    if (!canUpdate) return;
    QTimer::singleShot(DELAY_BUG, this, SLOT(onTimerFinished()));
    canUpdate=false;
    if (i==0) {
        videoWidget->setSaturation(0);
        return;
    }
    videoWidget->setSaturation( (qreal(i)/100.0) );
}
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
adjustAudio::adjustAudio(Phonon::AudioOutput *ao, QRect r, QWidget *parent) :
    QWidget(parent, /*Qt::FramelessWindowHint | */Qt::WindowStaysOnTopHint)
{
    volumeSlider = new Phonon::VolumeSlider(this);
    volumeSlider->setObjectName(QString::fromUtf8("volumeSlider"));
    this->setGeometry(r);
    this->setMinimumSize(r.size());
    this->setMaximumSize(r.size());
    volumeSlider->setMinimumSize(r.size());
    volumeSlider->setStyleSheet("");
    volumeSlider->setMaximumSize(r.size());
    volumeSlider->setOrientation(Qt::Vertical);
    volumeSlider->setAudioOutput(ao);
    this->show();
}

adjustAudio::~adjustAudio()
{
    delete volumeSlider;
}

void adjustAudio::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    painter.fillRect( event->rect(), Qt::black );
    painter.setPen(Qt::yellow);
    painter.drawRect( event->rect().x(), event->rect().y(),
                      event->rect().width()-1, event->rect().height()-1);
}

#endif // PHONON_LIB
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#ifdef PHONON_LIB
phononCapabilities mediaPlayer::phonon_capabilities={false,false};
#endif

mediaPlayer::mediaPlayer(bool accept_double_click, imageContainerKind container_kind, QWidget *parent) :
    QWidget(parent)
{
    if (/*(container_kind & CONTAINER_KIND_VIDEO) &&
            */MBbuffer::singleton) { qDebug() << "MBuffer pending to close"; this->close(); this->deleteLater();}
//    qDebug() << "MBbuffer::singleton" << MBbuffer::singleton;

    parentWidget=parent;
    acceptDoubleClick=accept_double_click;

    this->setWindowTitle(qApp->applicationName());
    horizontalLayout_2 = new QHBoxLayout(this);
    horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
    horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));

    kind=container_kind;
    this->setupUi();

    setAttribute(Qt::WA_DeleteOnClose);

    this->resize(QSize(0,0));
    resizing=false;
    isFullscreen=false;
}


mediaPlayer::~mediaPlayer()
{
    if (!pixContainer.isNull()) delete pixContainer;
#ifdef PHONON_LIB
    if ( (kind & CONTAINER_KIND_VIDEO) && !videoPlayer.isNull()) {
        videoPlayer->stop();
//        videoPlayer->mediaObject()->clear();
        delete PB_close;

        delete seekSlider;
        videoPlayer->deleteLater();
#ifndef MOBILE_APP
        delete volumeSliderButton;
        if (volumeSlider!=0) delete volumeSlider;
        delete PB_adjustVideo;
        if (adjustVideoWidget!=0) delete adjustVideoWidget;
        delete PB_fullscreen;
        if (!CB_formats.isNull()) delete CB_formats;
#endif
        if (!buffer.isNull()) buffer->deleteLater();
        delete L_time;
        if (!PB_info.isNull()) delete PB_info;
        delete horizontalLayout;
        delete verticalLayout;
    }
#endif
    delete horizontalLayout_2;
}

void mediaPlayer::paintEvent ( QPaintEvent * event )
{
    QPainter painter(this);
    painter.fillRect( event->rect(), Qt::black );
}
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
void mediaPlayer::setupUi()
{
    if (kind & CONTAINER_KIND_IMAGE)
    {
#ifdef PHONON_LIB
        if (!videoPlayer.isNull()) {
            videoPlayer->stop();
            videoPlayer->mediaObject()->clear();
            delete PB_close;
            delete seekSlider;
#ifndef MOBILE_APP
            delete volumeSliderButton;
            if (volumeSlider!=0) delete volumeSlider;
            delete PB_adjustVideo;
            if (adjustVideoWidget!=0) delete adjustVideoWidget;
            delete PB_fullscreen;
            if (!CB_formats.isNull()) delete CB_formats;
#endif
            delete videoPlayer;
            delete PB_playPause;
            if (!PB_info.isNull()) delete PB_info;
            delete L_time;
            delete horizontalLayout;
            delete verticalLayout;
        }
#endif
#ifndef MOBILE_APP
        pixContainer = new clickableLabel(MainWindow::pointerToMainWindow);
#else
        pixContainer = new clickableLabel(0);
#endif
        if (acceptDoubleClick)
            connect(pixContainer, SIGNAL(doubleClicked()), this, SLOT(close()));
    //        connect(pixContainer, SIGNAL(keyPressed(QKeyEvent*)), this, SLOT(onKeyPressed(QKeyEvent*)) );
        horizontalLayout_2->addWidget(pixContainer);
    }
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO)
    {
        if (!pixContainer.isNull()) delete pixContainer;
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(2);
        horizontalLayout->setContentsMargins(2, 2, 2, 2);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetNoConstraint);
        verticalLayout->addLayout(horizontalLayout);
        horizontalLayout_2->addLayout(verticalLayout);

        if (kind==CONTAINER_KIND_VIDEO)  // info button only for youTube videos
        {
            PB_info = new QPushButton;
            PB_info->setObjectName(QString::fromUtf8("PB_info"));
            PB_info->setIcon(QIcon(":/icons/icons/info_green.png"));
            horizontalLayout->addWidget(PB_info);
            connect(PB_info, SIGNAL(clicked()), this, SLOT(popupInfo()));
        }

        L_time = new QLabel;
        L_time->setObjectName(QString::fromUtf8("L_time"));
        L_time->setMinimumSize(QSize(75, 32));
        L_time->setText("");
        L_time->setFrameShape(QFrame::Panel);
        horizontalLayout->addWidget(L_time);

#ifndef MOBILE_APP
        if (kind==CONTAINER_KIND_VIDEO)
        {
            CB_formats = new QComboBox;
            CB_formats->setObjectName(QString::fromUtf8("CB_formats"));
            CB_formats->setMinimumSize(QSize(160, 32));
            horizontalLayout->addWidget(CB_formats);
        }

        PB_fullscreen = new QPushButton;
        PB_fullscreen->setObjectName(QString::fromUtf8("PB_info"));
        PB_fullscreen->setMinimumSize(QSize(32, 32));
        PB_fullscreen->setMaximumSize(QSize(32, 32));
        PB_fullscreen->setIcon(QIcon(":/icons/icons/fullscreen.png"));
        PB_fullscreen->setIconSize(QSize(32, 32));
        horizontalLayout->addWidget(PB_fullscreen);
        connect(PB_fullscreen, SIGNAL(clicked()), this, SLOT(switchFullScreen()));
#endif

        videoPlayer = new Phonon::VideoPlayer(Phonon::VideoCategory);
        videoPlayer->setObjectName(QString::fromUtf8("videoPlayer"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(10);
        sizePolicy.setHeightForWidth(videoPlayer->sizePolicy().hasHeightForWidth());
        videoPlayer->setSizePolicy(sizePolicy);
        videoPlayer->setMinimumSize(QSize(75, 75));
        videoPlayer->setStyleSheet("background: rgb(0,0,0);");
        verticalLayout->addWidget(videoPlayer);
        mediaObject=videoPlayer->mediaObject();
        mediaObject->setTickInterval(100); // default 1000
        connect(videoPlayer, SIGNAL(finished()), this, SLOT(onVideoPlayFinished()) );
        connect(mediaObject, SIGNAL(tick(qint64)), this, SLOT(tick(qint64)));
        connect(mediaObject, SIGNAL(bufferStatus(int)), this, SLOT(bufferChanged(int)) );
        connect(mediaObject, SIGNAL(stateChanged(Phonon::State, Phonon::State)),
                this, SLOT(phononStateChanged(Phonon::State, Phonon::State)));

        PB_playPause = new QPushButton;
        PB_playPause->setObjectName(QString::fromUtf8("PB_playPause"));
        PB_playPause->setIcon(PB_playPause->style()->standardIcon(QStyle::SP_MediaPause));
        horizontalLayout->addWidget(PB_playPause);
        connect(PB_playPause, SIGNAL(clicked()), this, SLOT(switchPlayPause()));

        seekSlider = new MBSlider;
        seekSlider->setObjectName(QString::fromUtf8("seekSlider"));
        seekSlider->setMinimumSize(QSize(170, 32));
        seekSlider->setMaximumSize(QSize(16777215, 16777215));
        seekSlider->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        connect(seekSlider, SIGNAL(sliderMoved(int)), this, SLOT(seekSliderChanged(int)) );
        horizontalLayout->addWidget(seekSlider);

#ifndef MOBILE_APP
        PB_adjustVideo = new QPushButton(this);
        PB_adjustVideo->setIcon(QIcon(":/icons/icons/graph.png"));
        connect(PB_adjustVideo, SIGNAL(clicked()), this, SLOT(onAdjustVideo_clicked()));
        horizontalLayout->addWidget(PB_adjustVideo);
        adjustVideoWidget=0;

        volumeSliderButton = new QPushButton;
        volumeSliderButton->setIcon(volumeSliderButton->style()->standardIcon(QStyle::SP_MediaVolume));
        connect(volumeSliderButton, SIGNAL(clicked()), this, SLOT(onVolumeSliderButton_clicked()));
        horizontalLayout->addWidget(volumeSliderButton);
        volumeSlider=0;
#endif

        PB_close = new QPushButton;
        PB_close->setObjectName(QString::fromUtf8("PB_close"));
        PB_close->setMinimumSize(QSize(50, 23));
        PB_close->setText(tr("close"));
        horizontalLayout->addWidget(PB_close);
        connect(PB_close, SIGNAL(clicked()), this, SLOT(close()));

        this->setStyleSheet("QLabel {color: rgb(255,255,0); background-color: rgb(0,0,0);} "
                            "QSlider { margin: 0px; background-color: rgb(120,120,120);}");

    }
#endif // PHONON_LIB
    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
    this->activateWindow();
}

#ifdef PHONON_LIB
void mediaPlayer::onAdjustVideo_clicked()
{
    if (adjustVideoWidget!=0) {
        delete adjustVideoWidget;
        adjustVideoWidget=0;
    }
    else {
        adjustVideoWidget = new adjustVideoParameters(videoPlayer->videoWidget(),
                                                      QRect(PB_adjustVideo->x()-300,
                                                            PB_adjustVideo->y()+PB_adjustVideo->height(),
                                                            300,130),
                                                      this);
        adjustVideoWidget->show();
    }
}

void mediaPlayer::onVolumeSliderButton_clicked()
{
    float Dwidth;
#ifdef MOBILE_APP
    Dwidth=1.5;
#else
    Dwidth=1;
#endif
    if (volumeSlider!=0) {
        delete volumeSlider;
        volumeSlider=0;
    }
    else
        volumeSlider = new adjustAudio(videoPlayer->audioOutput(),
                                       QRect(volumeSliderButton->geometry().x(),volumeSliderButton->geometry().y()+volumeSliderButton->height(),
                                             volumeSliderButton->geometry().width()*Dwidth, this->height()/4*3 ),
                                       this);
}
#endif

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////
// Handle KeyPress event for both mediaPlayer or pixContainer objects
void mediaPlayer::keyPressEvent( QKeyEvent * event )
{
    if (event->key() == Qt::Key_Escape) {
        this->close();
        emit keyPressed(event);
        return;
    }
    if (event->key() == Qt::Key_F11) {this->switchFullScreen();return;}
    if (event->key() == Qt::Key_Space && (kind & CONTAINER_KIND_VIDEO)) {this->switchPlayPause();return;}
    emit keyPressed(event);
}

bool mediaPlayer::event(QEvent *e)
{
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO) {
        switch(e->type())
        {
        case QEvent::MouseMove:
            if (!isFullscreen) break;
#ifndef MOBILE_APP
#ifndef QT_NO_CURSOR
            unsetCursor();
#endif
            for (int i = 0; i < horizontalLayout->count(); ++i)
                horizontalLayout->itemAt(i)->widget()->show();
            if (adjustVideoWidget!=0) adjustVideoWidget->show();
            if (volumeSlider!=0)      volumeSlider->show();
#endif  // MOBILE_APP
        case QEvent::WindowStateChange:
            {
                if (windowState() & Qt::WindowFullScreen) {
#ifndef MOBILE_APP
                    timer.start(1500, this);
#else
                    hideControls();
#endif
                }
                else {
                    timer.stop();
                    for (int i = 0; i < horizontalLayout->count(); ++i)
                        horizontalLayout->itemAt(i)->widget()->show();
#ifndef MOBILE_APP
                    if (adjustVideoWidget!=0) adjustVideoWidget->show();
                    if (volumeSlider!=0)      volumeSlider->show();
#endif

#ifndef QT_NO_CURSOR
                    unsetCursor();
#endif
                }
            }
            break;
        case QEvent::MouseButtonDblClick:
            if (windowState() & Qt::WindowFullScreen) goNormalScreen();
            else goFullScreen();
            break;
        default:
            break;
        }
    }
#endif  // PHONON_LIB
    //    return Phonon::VideoWidget::event(e);
    return QWidget::event(e);
}


void mediaPlayer::timerEvent(QTimerEvent *e)
{
#ifdef PHONON_LIB
    if (e->timerId() == timer.timerId()) {
        if (adjustVideoWidget==0 && volumeSlider==0) {
            hideControls();
        }
    }
#endif
    //    Phonon::VideoWidget::timerEvent(e);
    QWidget::timerEvent(e);
}

#ifdef PHONON_LIB
void mediaPlayer::hideControls()
{
    for (int i = 0; i < horizontalLayout->count(); ++i)
        horizontalLayout->itemAt(i)->widget()->hide();
#ifndef MOBILE_APP
    if (adjustVideoWidget!=0) adjustVideoWidget->hide();
    if (volumeSlider!=0)      volumeSlider->hide();
#endif
#ifndef QT_NO_CURSOR
    setCursor(Qt::BlankCursor);
#endif
}
#endif

//void mediaPlayer::onKeyPressed( QKeyEvent * event )
//{
//    keyPressEvent(event);
////    emit keyPressed(event);
//}



void mediaPlayer::goFullScreen()
{
    if (isFullscreen) return;
#ifndef MOBILE_APP
    this->setParent(0);
    this->setWindowFlags(Qt::FramelessWindowHint | Qt::Widget | Qt::WindowStaysOnTopHint);
    this->setGeometry(QApplication::desktop()->screenGeometry(-1));
    this->showFullScreen();
#ifdef PHONON_LIB
    if (!videoPlayer.isNull()) videoPlayer->setMouseTracking(true);
#endif
    this->setMouseTracking(true);

    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
    this->activateWindow();
#else
    this->setParent(0);
    this->showFullScreen();
    this->setFocus();
#endif

    isFullscreen=true;
    emit stateChanged();
}

void mediaPlayer::goNormalScreen()
{
//    this->setWindowFlags(Qt::FramelessWindowHint | Qt::Widget | Qt::WindowStaysOnTopHint);
    this->show();
#ifndef MOBILE_APP
    this->setParent(MainWindow::pointerToMainWindow);
    this->showNormal();
    this->setFocusPolicy(Qt::StrongFocus);
    this->setFocus();
#endif

//    this->setGeometry(QRect(0,0, parentWidget->width(),parentWidget->height()));
    this->setGeometry(QRect(0,0, MainWindow::pointerToMainWindow->width(),MainWindow::pointerToMainWindow->height()));
    this->activateWindow();

#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO) {
        for (int i = 0; i < horizontalLayout->count(); ++i)
            horizontalLayout->itemAt(i)->widget()->show();

#ifndef MOBILE_APP
        timer.stop();
        if (adjustVideoWidget!=0) adjustVideoWidget->show();
        if (volumeSlider!=0)      volumeSlider->show();
#endif
#ifndef QT_NO_CURSOR
        unsetCursor();
#endif

#ifdef Q_OS_SYMBIAN
    this->showFullScreen();
#elif defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6)
    this->showMaximized();
#endif

    }
#endif

//#else // MOBILE_APP

////    this->setParent(0);
////    this->setGeometry(QRect(0,0, 300,300));
//    this->setGeometry(QRect(0,0, MainWindow::pointerToMainWindow->width(),MainWindow::pointerToMainWindow->height()));
////    this->show();


//#endif

    isFullscreen=false;
    emit stateChanged();
}


void mediaPlayer::switchPlayPause()
{
    if (kind & CONTAINER_KIND_VIDEO) {
#ifdef PHONON_LIB
        if (videoPlayer->isPlaying()) {
            PB_playPause->setIcon(PB_playPause->style()->standardIcon(QStyle::SP_MediaPlay));
            videoPlayer->pause();
        } else
        {
            PB_playPause->setIcon(PB_playPause->style()->standardIcon(QStyle::SP_MediaPause));
            videoPlayer->play();
        }
#endif
    }
}

void mediaPlayer::switchFullScreen()
{
    QIcon icon;
    if (isFullscreen) {
        goNormalScreen();
        icon.addFile(QString::fromUtf8(":/icons/icons/fullscreen.png"), QSize(), QIcon::Normal, QIcon::Off);
    }
    else {
        goFullScreen();
        if (kind & CONTAINER_KIND_VIDEO) icon.addFile(QString::fromUtf8(":/icons/icons/nofullscreen.png"), QSize(), QIcon::Normal, QIcon::Off);
    }
#ifdef PHONON_LIB
    if (kind & CONTAINER_KIND_VIDEO) PB_fullscreen->setIcon(icon);
#endif
    // Set pixmap in the new format
    if (kind & CONTAINER_KIND_IMAGE) this->play(currPix);
}

// Size is the size of the container for the pixmap
void mediaPlayer::play(QPixmap &pixmap)
{
    QRect size;
    if (pixContainer.isNull()) this->setupUi();

    currPix = pixmap;
    if (isFullscreen) {
        size=QApplication::desktop()->screenGeometry(-1);
        goFullScreen();
    } else {
        size=parentWidget->geometry();
        goNormalScreen();
    }

    pixContainer->setGeometry(size);
    pixContainer->setStyleSheet("background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgb(38, 38, 194), stop:0.506726 rgb(0, 0, 0), stop:1 rgb(38, 38, 194));");
    pixContainer->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    pixContainer->setPixmap(pixmap.scaled(size.size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
//    pixContainer->setFocus();
//    pixContainer->activateWindow();
    pixContainer->show();
}

#ifdef PHONON_LIB
// Check if phonon backend can play mp4 and flv
void mediaPlayer::checkPhononCapabilities()
{
    QStringList sl=Phonon::BackendCapabilities::availableMimeTypes();

    phonon_capabilities.flv=sl.contains("video/x-flv") || sl.contains("video/x-flash-screen") ||
                sl.contains("video/x-flash-video") || sl.contains("video/quicktime");
    phonon_capabilities.mp4=sl.contains("video/x-h264") ||
                sl.contains("video/mp4") || sl.contains("video/mpg");
}

// Generic play with a given list of videos. Defaults play 1st video in the list of youtubeInfo
void mediaPlayer::play(YouTubeStruct t)
{
    if (t.video_info.size()==0) return;
    kind=CONTAINER_KIND_VIDEO;
    if (videoPlayer.isNull()) this->setupUi();
    curr_tube=t;
#ifndef MOBILE_APP
    CB_formats->disconnect(SIGNAL(currentIndexChanged(int)) , this);
    CB_formats->clear();
    CB_formats->setCurrentIndex(0);
    for (int i=0; i< t.video_info.size(); i++)
        CB_formats->addItem(t.video_info.at(i).fmt+" - "+t.video_info.at(i).humanFmt+" - "+t.video_info.at(i).container);
    CB_formats->setCurrentIndex(t.ID_preferred_video_format);
    connect(CB_formats, SIGNAL(currentIndexChanged(int)) , this, SLOT(play(int)));
#endif
    play( t.ID_preferred_video_format );
}

void mediaPlayer::play(int i)
{
    if (i>curr_tube.video_info.size()) return;
    curr_playing_video=i;

//    play(curr_tube.video_info.at(i).url_play);

    if (videoPlayer.isNull()) this->setupUi();
    if (videoPlayer->isPlaying()) {
        videoPlayer->stop();
    }
    videoPlayer->update();
    videoPlayer->repaint();

    qDebug() << "Playing: " << curr_tube.video_info.at(i).url_play;
    if (!buffer.isNull()) {
//        QEventLoop q;
//        connect(videoPlayer.data(), SIGNAL(finished()), &q, SLOT(quit()) );
//        QTimer::singleShot(50, videoPlayer.data(), SLOT(stop()) );
//        if (videoPlayer->isPlaying()) q.exec(); // Wait till player stops
//        qDebug() << "videoPlayer STOPPED";
//        seekSlider->clearBufferChunks();
//        connect(buffer.data(), SIGNAL(destroyed()), &q, SLOT(quit()) );
////        delete buffer;
//        QTimer::singleShot(50, buffer.data(), SLOT(deleteLater()) );
//        q.exec(); // Wait till buffer is deleted
//        qDebug() << "buffer DELETED";

        buffer->close();
        buffer->init(QUrl(curr_tube.video_info.at(i).url_play));
        seekSlider->clearBufferChunks();
    }
    else {
        buffer = new MBbuffer(QUrl(curr_tube.video_info.at(i).url_play), network::cookieJar);
        connect(buffer, SIGNAL(bufferEmptyed()), videoPlayer->mediaObject(), SLOT(pause()), Qt::DirectConnection  );
        connect(buffer, SIGNAL(bufferReady()), videoPlayer->mediaObject(), SLOT(play()), Qt::DirectConnection  );
        connect(buffer, SIGNAL(dataWritten(int,int)), seekSlider, SLOT(setBufferChunks(int,int)) );
    }

    seekSlider->setValue(0);
    connect(seekSlider, SIGNAL(sliderMoved(int)), this, SLOT(seekSliderChanged(int)) );

    buffer->startDownload();
    videoPlayer->mediaObject()->setCurrentSource(Phonon::MediaSource(buffer));
    videoPlayer->play();

    this->activateWindow();
    this->setFocus();
}

// Play specific video url
void mediaPlayer::play(QString url)
{
    if (videoPlayer.isNull()) this->setupUi();
    if (videoPlayer->isPlaying()) {
        videoPlayer->stop();
    }
    videoPlayer->update();
    videoPlayer->repaint();
    // Withouot opengl seems that the 1st time playing video, the widget
    // isn't updated. Pressing fullscreen and restore works
#ifndef QT_OPENGL_SUPPORT
    if (isFullscreen) goNormalScreen();
    else goFullScreen();
    if (isFullscreen) goNormalScreen();
    else goFullScreen();
#endif

    qDebug() << "Playing: " << url;

//    seekSlider->setMediaObject(mediaObject);

//    this->show();
    videoPlayer->play(url);

    this->activateWindow();
    this->setFocus();
}

void mediaPlayer::phononStateChanged(Phonon::State newstate, Phonon::State oldstate)
{
    static int retry=0;
    if (newstate == Phonon::ErrorState) {
        retry++;
        qDebug() << "******************** newstate" << newstate << "oldstate" << oldstate << "retry" << retry;
        // retry 3 times after giving up
        if (oldstate == Phonon::LoadingState && retry < 3) {
            play(curr_tube);
            return;
        }
        retry=0;

        if (mediaObject->errorType() == Phonon::FatalError) {
            QMessageBox::warning(0, "Phonon Mediaplayer",
                                 mediaObject->errorString(),
                                 QMessageBox::Close);
            this->close();
        } else {
            stopPlay();
        }
    }
    if (newstate == Phonon::PlayingState) retry = 0;
}

bool mediaPlayer::isPlaying()
{
    return videoPlayer->isPlaying();
}

void mediaPlayer::stopPlay()
{
    ticks=0;
    videoPlayer->stop();
}

void mediaPlayer::bufferChanged(int i)
{
    qDebug() << "Buffer: " << i;
}

void mediaPlayer::seekableChanged(bool b)
{
    Q_UNUSED(b);

//    qDebug() << "Seekable: " << b;
//    seekSlider->setTracking(false);
//    qDebug() << "HasTracking: " << seekSlider->hasTracking();
}

void mediaPlayer::seekSliderChanged(int value)
{
    if (kind!=CONTAINER_KIND_VIDEO_LOCAL) {
        if (buffer.isNull()) return;
        if (!videoPlayer->mediaObject()->isSeekable()) return;
        if ( !buffer.isNull() && buffer->isBuffering()) return;
    }
    qint64 newValue;
    QEventLoop loop;

    newValue = (qint64)videoPlayer->mediaObject()->totalTime()*value/1000.0;
    videoPlayer->mediaObject()->seek( newValue );

    if ( !buffer.isNull() ) {
        connect(buffer, SIGNAL(readingData(qint64,qint64)), &loop, SLOT(quit()) );
        loop.exec();

        qint64 r = buffer->readPosValue();
        if (!buffer->readPosInRange(r)) {
            seekSlider->clearBufferChunks();
        }
        buffer->startDownloadFromBytePos(r);
    }
}

void mediaPlayer::tick(qint64 time)
{
    if (time <= 0) return;
    ticks=mediaObject->remainingTime();

    int percStreamed = 1000.0*mediaObject->currentTime()/mediaObject->totalTime();
//    qDebug() << "percStreamed" << percStreamed;
    seekSlider->setValue(percStreamed);

    L_time->setText(QString::number(ticks/1000)+"/"+QString::number(mediaObject->totalTime()/1000));
}

void mediaPlayer::popupInfo()
{
    QString s;
    s="<font color=#ffffff size=4><p>"+curr_tube.duration+"\" "+
      curr_tube.video_info.at(curr_playing_video).container+" - "+curr_tube.video_info.at(curr_playing_video).humanFmt+"</p>"+
      "<p>"+curr_tube.published+" by "+curr_tube.author_name+"</p>"+
      "<p>Viewed: "+curr_tube.statistics_viewCount+" Rating: "+curr_tube.rating_average+" with "+curr_tube.rating_numRaters+" votes </p>"+
      "<p><h3><strong>"+curr_tube.title+"</strong></h3></p>"+
      curr_tube.content+
      "</font>";

    popupText *p=new popupText(this);
      int w,h;
#ifndef MOBILE_APP
      w=this->width()/3*2;
      h=this->height()/3*2;
#else
      w=this->width();
      h=this->height();
#endif
#ifdef Q_OS_SYMBIAN
    this->showFullScreen();
#endif
      p->showHtmlText(s, QRect(this->PB_info->pos().x(), this->PB_info->y(),
                               w, h )
                      );
}

void mediaPlayer::onVideoPlayFinished()
{
    ticks=0;
    emit videoPlayFinished();
}

#endif


