/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef MEDIAPLAYER_H
#define MEDIAPLAYER_H

#include <QWidget>
#include <QTimer>
#include <QLabel>
#include <QHBoxLayout>
#include <QProgressBar>
#if QT_VERSION > 0x040600
    #include <QPropertyAnimation>
#endif
#include <QPushButton>
#include <QPointer>
#ifdef PHONON_LIB
    #include <phonon/mediaobject.h>
    #include <phonon/videowidget.h>
    #include <phonon/audiooutput.h>
    #include <phonon/seekslider.h>
    #include <phonon/volumeslider.h>
    #include <phonon/videoplayer.h>
#endif
#include <QtGui/QComboBox>
#include "network.h"
#include "youtube.h"
#include "defines.h"
#include "mbslider.h"
#include "mbbuffer.h"

class clickableLabel;
class mediaPlayer;

typedef struct {
   bool flv;
   bool mp4;
} phononCapabilities ;


// Class to display pixmaps
class clickableLabel : public QLabel
{
    Q_OBJECT

public:
    clickableLabel(QWidget *parent = 0);

protected:
    virtual void mousePressEvent ( QMouseEvent *e );
    virtual void mouseDoubleClickEvent ( QMouseEvent * e );
//    virtual void mouseReleaseEvent ( QMouseEvent *e );

signals:
    void clicked();
    void doubleClicked();
    void keyPressed( QKeyEvent * event );
};
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
#ifdef PHONON_LIB
// Widget to adjust video saturation, contrast, brightness
class adjustVideoParameters : public QWidget
{
    Q_OBJECT

public:
    explicit adjustVideoParameters(Phonon::VideoWidget * vw, QRect r, QWidget *parent);
    ~adjustVideoParameters();

private:
    bool canUpdate;
    QGridLayout *layout;
    Phonon::VideoWidget *videoWidget;
    QSlider *brightness;
    QSlider *contrast;
    QSlider *hue;
    QSlider *saturation;
    QLabel  *brightnessLabel;
    QLabel  *contrastLabel;
    QLabel  *hueLabel;
    QLabel  *saturationLabel;
    QPushButton *PB_reset;

protected:
    virtual void paintEvent ( QPaintEvent * event );

public slots:
    void onPB_reset_clicked();
    void onBrightness_moved(int i);
    void onContrast_moved(int i);
    void onHue_moved(int i);
    void onSaturation_moved(int i);
    void onTimerFinished();
};
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
class adjustAudio : public QWidget
{
    Q_OBJECT

public:
    explicit adjustAudio(Phonon::AudioOutput *ao, QRect r, QWidget *parent=0);
    ~adjustAudio();

private:
    Phonon::VolumeSlider *volumeSlider;

protected:
    virtual void paintEvent ( QPaintEvent * event );
};
#endif // PHONON_LIB
/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////
class mediaPlayer : public QWidget
{
    Q_OBJECT

public:
    explicit mediaPlayer(bool accept_doubleClick, imageContainerKind container_kind, QWidget *parent = 0);
    ~mediaPlayer();
    void play(QPixmap &pixmap);

#ifdef PHONON_LIB
    void play(YouTubeStruct tubes);
    void play(QString url);
    void stopPlay();
    bool isPlaying();
    static void checkPhononCapabilities();
    static phononCapabilities phonon_capabilities;
#endif

    YouTubeStruct curr_tube;
    QSize curr_video_size;
    imageContainerKind kind;
    QPointer<clickableLabel> pixContainer;
    QPixmap currPix;
    qint64 ticks;
    bool isFullscreen;

private:
    void setupUi();
    QWidget *parentWidget;

    QHBoxLayout *horizontalLayout_2;
#ifdef PHONON_LIB
    QPointer<MBbuffer>   buffer;
    MBSlider             *seekSlider;
    QPushButton          *volumeSliderButton;
    adjustAudio          *volumeSlider;
    QVBoxLayout          *verticalLayout;
    QHBoxLayout          *horizontalLayout;
    QPushButton          *PB_adjustVideo;
    adjustVideoParameters *adjustVideoWidget;
    QPointer<QComboBox>   CB_formats;
    QPushButton          *PB_close;
    QPointer<QPushButton> PB_info;
    QPushButton          *PB_playPause;
    QPushButton          *PB_fullscreen;
    QLabel               *L_time;
    quint64 currentStreamDownload;
    quint64 currentStreamDownloadLength;
    quint64 percStreamDownloaded;

    QPointer<Phonon::VideoPlayer> videoPlayer;
    Phonon::MediaObject *mediaObject;
    Phonon::AudioOutput *audioOutput;
    Phonon::MediaSource *mediaSource;
    QList<youTubeVideoInfo> videoInfo;
    void hideControls();
#endif
    bool resizing;
    int curr_playing_video;
    bool acceptDoubleClick;
    QBasicTimer timer;

protected:
    void keyPressEvent( QKeyEvent * event );
    bool event(QEvent *e);
    void timerEvent(QTimerEvent *e);
    void paintEvent ( QPaintEvent * event );

public slots:
    void switchFullScreen();
    void goFullScreen();
    void goNormalScreen();
    void switchPlayPause();
#ifdef PHONON_LIB
    void play(int i);
    void popupInfo();
    void onVideoPlayFinished();
    void onVolumeSliderButton_clicked();
    void onAdjustVideo_clicked();

private slots:
    void tick(qint64 time);
    void seekableChanged(bool b);
    void seekSliderChanged(int value);
    void bufferChanged(int i);
    void phononStateChanged(Phonon::State newstate, Phonon::State oldstate);

signals:
    void videoFormatChanged(QSize size);
    void videoPlayFinished();
#endif

//public slots:
//    void onKeyPressed( QKeyEvent * event );
signals:
    void keyPressed( QKeyEvent * event );
    void stateChanged();
};

#endif // MEDIAPLAYER_H
