/***************************************************************************
*  Media Downloader. Search and download as many images or videos that     *
*    fit your search criteria                                              *
*  Copyright (C) 2009-2011  Marco Bavagnoli - lil.deimos@gmail.com         *
*                                                                          *
*  This program is free software: you can redistribute it and/or modify    *
*  it under the terms of the GNU General Public License as published by    *
*  the Free Software Foundation, either version 3 of the License, or       *
*  (at your option) any later version.                                     *
*                                                                          *
*  This program is distributed in the hope that it will be useful,         *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*  GNU General Public License for more details.                            *
*                                                                          *
*  You should have received a copy of the GNU General Public License       *
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
***************************************************************************/

#ifndef SAVE_H
#define SAVE_H

#include <QObject>
#include <QByteArray>
#include <QFile>
#include <QFileInfo>
#include "network.h"
#include "imagecontainer.h"
#include "googlepicture.h"
#include "youtube.h"
#ifndef MOBILE_APP
#include "ffmpegfrontend.h"
#endif

class save : public QObject
{
    Q_OBJECT
public:
    enum save_options
    {
        USE_NUMBER_PLUS_FILENAME,
        USE_NUMBER,
        USE_FILENAME
    };

    save(QDialog *parent = 0);
    QFileInfo save_to_local_file(QByteArray buffer,
                                 imageStruct item,
                                 int n,
                                 QString default_ext);
    QFileInfo download_one(imageStruct item,
                           QString local_dir,
                           save_options opt,
                           int start,
                           bool convert,
                           bool createSearchDir);
    void download_all(QList<imageStruct> items,
                      QString local_dir,
                      save_options opt,
                      int start,
                      bool convert,
                      bool createSearchDir);
    static network *net_async;
#ifndef MOBILE_APP
    ffmpegFrontend *ffmpegDialog;
#endif

private:
    QString checkFileName(QString s);
    save_options options_for_save;
    QString dir_to_save;
    int start_image_number;
    bool convert_video;
    bool create_search_dir;
    network *net;
    QFile *file;
    QList<imageStruct> items_to_download;
    QStringList default_extensions;

public slots:
    void downloaded(int n, QByteArray buffer);
    void allDownloaded();

signals:
    void downloadsComplete();
};

#endif // DOWNLOAD_H
